/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Virtual Machine Scale Set OS Upgrade History operation response.
 *
 */
class UpgradeOperationHistoricalStatusInfo {
  /**
   * Create a UpgradeOperationHistoricalStatusInfo.
   * @member {object} [properties] Information about the properties of the
   * upgrade operation.
   * @member {object} [properties.runningStatus] Information about the overall
   * status of the upgrade operation.
   * @member {string} [properties.runningStatus.code] Code indicating the
   * current status of the upgrade. Possible values include: 'RollingForward',
   * 'Cancelled', 'Completed', 'Faulted'
   * @member {date} [properties.runningStatus.startTime] Start time of the
   * upgrade.
   * @member {date} [properties.runningStatus.endTime] End time of the upgrade.
   * @member {object} [properties.progress] Counts of the VM's in each state.
   * @member {number} [properties.progress.successfulInstanceCount] The number
   * of instances that have been successfully upgraded.
   * @member {number} [properties.progress.failedInstanceCount] The number of
   * instances that have failed to be upgraded successfully.
   * @member {number} [properties.progress.inProgressInstanceCount] The number
   * of instances that are currently being upgraded.
   * @member {number} [properties.progress.pendingInstanceCount] The number of
   * instances that have not yet begun to be upgraded.
   * @member {object} [properties.error] Error Details for this upgrade if
   * there are any.
   * @member {array} [properties.error.details] The Api error details
   * @member {object} [properties.error.innererror] The Api inner error
   * @member {string} [properties.error.innererror.exceptiontype] The exception
   * type.
   * @member {string} [properties.error.innererror.errordetail] The internal
   * error message or exception dump.
   * @member {string} [properties.error.code] The error code.
   * @member {string} [properties.error.target] The target of the particular
   * error.
   * @member {string} [properties.error.message] The error message.
   * @member {string} [properties.startedBy] Invoker of the Upgrade Operation.
   * Possible values include: 'Unknown', 'User', 'Platform'
   * @member {object} [properties.targetImageReference] Image Reference details
   * @member {string} [properties.targetImageReference.publisher] The image
   * publisher.
   * @member {string} [properties.targetImageReference.offer] Specifies the
   * offer of the platform image or marketplace image used to create the
   * virtual machine.
   * @member {string} [properties.targetImageReference.sku] The image SKU.
   * @member {string} [properties.targetImageReference.version] Specifies the
   * version of the platform image or marketplace image used to create the
   * virtual machine. The allowed formats are Major.Minor.Build or 'latest'.
   * Major, Minor, and Build are decimal numbers. Specify 'latest' to use the
   * latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time
   * even if a new version becomes available.
   * @member {object} [properties.rollbackInfo] Information about OS rollback
   * if performed
   * @member {number}
   * [properties.rollbackInfo.successfullyRolledbackInstanceCount] The number
   * of instances which have been successfully rolled back.
   * @member {number} [properties.rollbackInfo.failedRolledbackInstanceCount]
   * The number of instances which failed to rollback.
   * @member {object} [properties.rollbackInfo.rollbackError] Error details if
   * OS rollback failed.
   * @member {array} [properties.rollbackInfo.rollbackError.details] The Api
   * error details
   * @member {object} [properties.rollbackInfo.rollbackError.innererror] The
   * Api inner error
   * @member {string}
   * [properties.rollbackInfo.rollbackError.innererror.exceptiontype] The
   * exception type.
   * @member {string}
   * [properties.rollbackInfo.rollbackError.innererror.errordetail] The
   * internal error message or exception dump.
   * @member {string} [properties.rollbackInfo.rollbackError.code] The error
   * code.
   * @member {string} [properties.rollbackInfo.rollbackError.target] The target
   * of the particular error.
   * @member {string} [properties.rollbackInfo.rollbackError.message] The error
   * message.
   * @member {string} [type] Resource type
   * @member {string} [location] Resource location
   */
  constructor() {
  }

  /**
   * Defines the metadata of UpgradeOperationHistoricalStatusInfo
   *
   * @returns {object} metadata of UpgradeOperationHistoricalStatusInfo
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'UpgradeOperationHistoricalStatusInfo',
      type: {
        name: 'Composite',
        className: 'UpgradeOperationHistoricalStatusInfo',
        modelProperties: {
          properties: {
            required: false,
            readOnly: true,
            serializedName: 'properties',
            type: {
              name: 'Composite',
              className: 'UpgradeOperationHistoricalStatusInfoProperties'
            }
          },
          type: {
            required: false,
            readOnly: true,
            serializedName: 'type',
            type: {
              name: 'String'
            }
          },
          location: {
            required: false,
            readOnly: true,
            serializedName: 'location',
            type: {
              name: 'String'
            }
          }
        }
      }
    };
  }
}

module.exports = UpgradeOperationHistoricalStatusInfo;
