/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

/**
 * Describes a virtual machine scale set OS profile.
 *
 */
class VirtualMachineScaleSetUpdateOSProfile {
  /**
   * Create a VirtualMachineScaleSetUpdateOSProfile.
   * @member {string} [customData] A base-64 encoded string of custom data.
   * @member {object} [windowsConfiguration] The Windows Configuration of the
   * OS profile.
   * @member {boolean} [windowsConfiguration.provisionVMAgent] Indicates
   * whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   * @member {boolean} [windowsConfiguration.enableAutomaticUpdates] Indicates
   * whether virtual machine is enabled for automatic updates.
   * @member {string} [windowsConfiguration.timeZone] Specifies the time zone
   * of the virtual machine. e.g. "Pacific Standard Time"
   * @member {array} [windowsConfiguration.additionalUnattendContent] Specifies
   * additional base-64 encoded XML formatted information that can be included
   * in the Unattend.xml file, which is used by Windows Setup.
   * @member {object} [windowsConfiguration.winRM] Specifies the Windows Remote
   * Management listeners. This enables remote Windows PowerShell.
   * @member {array} [windowsConfiguration.winRM.listeners] The list of Windows
   * Remote Management listeners
   * @member {object} [linuxConfiguration] The Linux Configuration of the OS
   * profile.
   * @member {boolean} [linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   * @member {object} [linuxConfiguration.ssh] Specifies the ssh key
   * configuration for a Linux OS.
   * @member {array} [linuxConfiguration.ssh.publicKeys] The list of SSH public
   * keys used to authenticate with linux based VMs.
   * @member {boolean} [linuxConfiguration.provisionVMAgent] Indicates whether
   * virtual machine agent should be provisioned on the virtual machine.
   * <br><br> When this property is not specified in the request body, default
   * behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   * @member {array} [secrets] The List of certificates for addition to the VM.
   */
  constructor() {
  }

  /**
   * Defines the metadata of VirtualMachineScaleSetUpdateOSProfile
   *
   * @returns {object} metadata of VirtualMachineScaleSetUpdateOSProfile
   *
   */
  mapper() {
    return {
      required: false,
      serializedName: 'VirtualMachineScaleSetUpdateOSProfile',
      type: {
        name: 'Composite',
        className: 'VirtualMachineScaleSetUpdateOSProfile',
        modelProperties: {
          customData: {
            required: false,
            serializedName: 'customData',
            type: {
              name: 'String'
            }
          },
          windowsConfiguration: {
            required: false,
            serializedName: 'windowsConfiguration',
            type: {
              name: 'Composite',
              className: 'WindowsConfiguration'
            }
          },
          linuxConfiguration: {
            required: false,
            serializedName: 'linuxConfiguration',
            type: {
              name: 'Composite',
              className: 'LinuxConfiguration'
            }
          },
          secrets: {
            required: false,
            serializedName: 'secrets',
            type: {
              name: 'Sequence',
              element: {
                  required: false,
                  serializedName: 'VaultSecretGroupElementType',
                  type: {
                    name: 'Composite',
                    className: 'VaultSecretGroup'
                  }
              }
            }
          }
        }
      }
    };
  }
}

module.exports = VirtualMachineScaleSetUpdateOSProfile;
