/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;


/**
 * Export logs that show Api requests made by this subscription in the given
 * time window to show throttling activities.
 *
 * @param {object} parameters Parameters supplied to the LogAnalytics
 * getRequestRateByInterval Api.
 *
 * @param {string} parameters.intervalLength Interval value in minutes used to
 * create LogAnalytics call rate logs. Possible values include: 'ThreeMins',
 * 'FiveMins', 'ThirtyMins', 'SixtyMins'
 *
 * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
 * container to which LogAnalytics Api writes output logs to.
 *
 * @param {date} parameters.fromTime From time of the query
 *
 * @param {date} parameters.toTime To time of the query
 *
 * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
 * Throttle Policy applied.
 *
 * @param {boolean} [parameters.groupByOperationName] Group query result by  by
 * Operation Name.
 *
 * @param {boolean} [parameters.groupByResourceName] Group query result by
 * Resource Name.
 *
 * @param {string} location The location upon which virtual-machine-sizes is
 * queried.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link LogAnalyticsOperationResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _exportRequestRateByInterval(parameters, location, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginExportRequestRateByInterval(parameters, location, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['LogAnalyticsOperationResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Export logs that show total throttled Api requests for this subscription in
 * the given time window.
 *
 * @param {object} parameters Parameters supplied to the LogAnalytics
 * getThrottledRequests Api.
 *
 * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
 * container to which LogAnalytics Api writes output logs to.
 *
 * @param {date} parameters.fromTime From time of the query
 *
 * @param {date} parameters.toTime To time of the query
 *
 * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
 * Throttle Policy applied.
 *
 * @param {boolean} [parameters.groupByOperationName] Group query result by  by
 * Operation Name.
 *
 * @param {boolean} [parameters.groupByResourceName] Group query result by
 * Resource Name.
 *
 * @param {string} location The location upon which virtual-machine-sizes is
 * queried.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link LogAnalyticsOperationResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _exportThrottledRequests(parameters, location, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginExportThrottledRequests(parameters, location, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['LogAnalyticsOperationResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }

      return callback(null, result, httpRequest, response);
    });
  });
}

/**
 * Export logs that show Api requests made by this subscription in the given
 * time window to show throttling activities.
 *
 * @param {object} parameters Parameters supplied to the LogAnalytics
 * getRequestRateByInterval Api.
 *
 * @param {string} parameters.intervalLength Interval value in minutes used to
 * create LogAnalytics call rate logs. Possible values include: 'ThreeMins',
 * 'FiveMins', 'ThirtyMins', 'SixtyMins'
 *
 * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
 * container to which LogAnalytics Api writes output logs to.
 *
 * @param {date} parameters.fromTime From time of the query
 *
 * @param {date} parameters.toTime To time of the query
 *
 * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
 * Throttle Policy applied.
 *
 * @param {boolean} [parameters.groupByOperationName] Group query result by  by
 * Operation Name.
 *
 * @param {boolean} [parameters.groupByResourceName] Group query result by
 * Resource Name.
 *
 * @param {string} location The location upon which virtual-machine-sizes is
 * queried.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link LogAnalyticsOperationResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginExportRequestRateByInterval(parameters, location, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (parameters === null || parameters === undefined) {
      throw new Error('parameters cannot be null or undefined.');
    }
    if (location === null || location === undefined || typeof location.valueOf() !== 'string') {
      throw new Error('location cannot be null or undefined and it must be of type string.');
    }
    if (location !== null && location !== undefined) {
      if (location.match(/^[-\w\._]+$/) === null)
      {
        throw new Error('"location" should satisfy the constraint - "Pattern": /^[-\w\._]+$/');
      }
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/logAnalytics/apiAccess/getRequestRateByInterval';
  requestUrl = requestUrl.replace('{location}', encodeURIComponent(location));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (parameters !== null && parameters !== undefined) {
      let requestModelMapper = new client.models['RequestRateByIntervalInput']().mapper();
      requestModel = client.serialize(requestModelMapper, parameters, 'parameters');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(parameters, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['LogAnalyticsOperationResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Export logs that show total throttled Api requests for this subscription in
 * the given time window.
 *
 * @param {object} parameters Parameters supplied to the LogAnalytics
 * getThrottledRequests Api.
 *
 * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
 * container to which LogAnalytics Api writes output logs to.
 *
 * @param {date} parameters.fromTime From time of the query
 *
 * @param {date} parameters.toTime To time of the query
 *
 * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
 * Throttle Policy applied.
 *
 * @param {boolean} [parameters.groupByOperationName] Group query result by  by
 * Operation Name.
 *
 * @param {boolean} [parameters.groupByResourceName] Group query result by
 * Resource Name.
 *
 * @param {string} location The location upon which virtual-machine-sizes is
 * queried.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link LogAnalyticsOperationResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginExportThrottledRequests(parameters, location, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (location === null || location === undefined || typeof location.valueOf() !== 'string') {
      throw new Error('location cannot be null or undefined and it must be of type string.');
    }
    if (location !== null && location !== undefined) {
      if (location.match(/^[-\w\._]+$/) === null)
      {
        throw new Error('"location" should satisfy the constraint - "Pattern": /^[-\w\._]+$/');
      }
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/providers/Microsoft.Compute/locations/{location}/logAnalytics/apiAccess/getThrottledRequests';
  requestUrl = requestUrl.replace('{location}', encodeURIComponent(location));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (parameters !== null && parameters !== undefined) {
      let requestModelMapper = new client.models['ThrottledRequestsInput']().mapper();
      requestModel = client.serialize(requestModelMapper, parameters, 'parameters');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(parameters, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['LogAnalyticsOperationResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a LogAnalytics. */
class LogAnalytics {
  /**
   * Create a LogAnalytics.
   * @param {ComputeManagementClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._exportRequestRateByInterval = _exportRequestRateByInterval;
    this._exportThrottledRequests = _exportThrottledRequests;
    this._beginExportRequestRateByInterval = _beginExportRequestRateByInterval;
    this._beginExportThrottledRequests = _beginExportThrottledRequests;
  }

  /**
   * Export logs that show Api requests made by this subscription in the given
   * time window to show throttling activities.
   *
   * @param {object} parameters Parameters supplied to the LogAnalytics
   * getRequestRateByInterval Api.
   *
   * @param {string} parameters.intervalLength Interval value in minutes used to
   * create LogAnalytics call rate logs. Possible values include: 'ThreeMins',
   * 'FiveMins', 'ThirtyMins', 'SixtyMins'
   *
   * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
   * container to which LogAnalytics Api writes output logs to.
   *
   * @param {date} parameters.fromTime From time of the query
   *
   * @param {date} parameters.toTime To time of the query
   *
   * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
   * Throttle Policy applied.
   *
   * @param {boolean} [parameters.groupByOperationName] Group query result by  by
   * Operation Name.
   *
   * @param {boolean} [parameters.groupByResourceName] Group query result by
   * Resource Name.
   *
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<LogAnalyticsOperationResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  exportRequestRateByIntervalWithHttpOperationResponse(parameters, location, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._exportRequestRateByInterval(parameters, location, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Export logs that show Api requests made by this subscription in the given
   * time window to show throttling activities.
   *
   * @param {object} parameters Parameters supplied to the LogAnalytics
   * getRequestRateByInterval Api.
   *
   * @param {string} parameters.intervalLength Interval value in minutes used to
   * create LogAnalytics call rate logs. Possible values include: 'ThreeMins',
   * 'FiveMins', 'ThirtyMins', 'SixtyMins'
   *
   * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
   * container to which LogAnalytics Api writes output logs to.
   *
   * @param {date} parameters.fromTime From time of the query
   *
   * @param {date} parameters.toTime To time of the query
   *
   * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
   * Throttle Policy applied.
   *
   * @param {boolean} [parameters.groupByOperationName] Group query result by  by
   * Operation Name.
   *
   * @param {boolean} [parameters.groupByResourceName] Group query result by
   * Resource Name.
   *
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {LogAnalyticsOperationResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link LogAnalyticsOperationResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  exportRequestRateByInterval(parameters, location, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._exportRequestRateByInterval(parameters, location, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._exportRequestRateByInterval(parameters, location, options, optionalCallback);
    }
  }

  /**
   * Export logs that show total throttled Api requests for this subscription in
   * the given time window.
   *
   * @param {object} parameters Parameters supplied to the LogAnalytics
   * getThrottledRequests Api.
   *
   * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
   * container to which LogAnalytics Api writes output logs to.
   *
   * @param {date} parameters.fromTime From time of the query
   *
   * @param {date} parameters.toTime To time of the query
   *
   * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
   * Throttle Policy applied.
   *
   * @param {boolean} [parameters.groupByOperationName] Group query result by  by
   * Operation Name.
   *
   * @param {boolean} [parameters.groupByResourceName] Group query result by
   * Resource Name.
   *
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<LogAnalyticsOperationResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  exportThrottledRequestsWithHttpOperationResponse(parameters, location, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._exportThrottledRequests(parameters, location, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Export logs that show total throttled Api requests for this subscription in
   * the given time window.
   *
   * @param {object} parameters Parameters supplied to the LogAnalytics
   * getThrottledRequests Api.
   *
   * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
   * container to which LogAnalytics Api writes output logs to.
   *
   * @param {date} parameters.fromTime From time of the query
   *
   * @param {date} parameters.toTime To time of the query
   *
   * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
   * Throttle Policy applied.
   *
   * @param {boolean} [parameters.groupByOperationName] Group query result by  by
   * Operation Name.
   *
   * @param {boolean} [parameters.groupByResourceName] Group query result by
   * Resource Name.
   *
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {LogAnalyticsOperationResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link LogAnalyticsOperationResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  exportThrottledRequests(parameters, location, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._exportThrottledRequests(parameters, location, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._exportThrottledRequests(parameters, location, options, optionalCallback);
    }
  }

  /**
   * Export logs that show Api requests made by this subscription in the given
   * time window to show throttling activities.
   *
   * @param {object} parameters Parameters supplied to the LogAnalytics
   * getRequestRateByInterval Api.
   *
   * @param {string} parameters.intervalLength Interval value in minutes used to
   * create LogAnalytics call rate logs. Possible values include: 'ThreeMins',
   * 'FiveMins', 'ThirtyMins', 'SixtyMins'
   *
   * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
   * container to which LogAnalytics Api writes output logs to.
   *
   * @param {date} parameters.fromTime From time of the query
   *
   * @param {date} parameters.toTime To time of the query
   *
   * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
   * Throttle Policy applied.
   *
   * @param {boolean} [parameters.groupByOperationName] Group query result by  by
   * Operation Name.
   *
   * @param {boolean} [parameters.groupByResourceName] Group query result by
   * Resource Name.
   *
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<LogAnalyticsOperationResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginExportRequestRateByIntervalWithHttpOperationResponse(parameters, location, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginExportRequestRateByInterval(parameters, location, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Export logs that show Api requests made by this subscription in the given
   * time window to show throttling activities.
   *
   * @param {object} parameters Parameters supplied to the LogAnalytics
   * getRequestRateByInterval Api.
   *
   * @param {string} parameters.intervalLength Interval value in minutes used to
   * create LogAnalytics call rate logs. Possible values include: 'ThreeMins',
   * 'FiveMins', 'ThirtyMins', 'SixtyMins'
   *
   * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
   * container to which LogAnalytics Api writes output logs to.
   *
   * @param {date} parameters.fromTime From time of the query
   *
   * @param {date} parameters.toTime To time of the query
   *
   * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
   * Throttle Policy applied.
   *
   * @param {boolean} [parameters.groupByOperationName] Group query result by  by
   * Operation Name.
   *
   * @param {boolean} [parameters.groupByResourceName] Group query result by
   * Resource Name.
   *
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {LogAnalyticsOperationResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link LogAnalyticsOperationResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginExportRequestRateByInterval(parameters, location, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginExportRequestRateByInterval(parameters, location, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginExportRequestRateByInterval(parameters, location, options, optionalCallback);
    }
  }

  /**
   * Export logs that show total throttled Api requests for this subscription in
   * the given time window.
   *
   * @param {object} parameters Parameters supplied to the LogAnalytics
   * getThrottledRequests Api.
   *
   * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
   * container to which LogAnalytics Api writes output logs to.
   *
   * @param {date} parameters.fromTime From time of the query
   *
   * @param {date} parameters.toTime To time of the query
   *
   * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
   * Throttle Policy applied.
   *
   * @param {boolean} [parameters.groupByOperationName] Group query result by  by
   * Operation Name.
   *
   * @param {boolean} [parameters.groupByResourceName] Group query result by
   * Resource Name.
   *
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<LogAnalyticsOperationResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginExportThrottledRequestsWithHttpOperationResponse(parameters, location, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginExportThrottledRequests(parameters, location, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Export logs that show total throttled Api requests for this subscription in
   * the given time window.
   *
   * @param {object} parameters Parameters supplied to the LogAnalytics
   * getThrottledRequests Api.
   *
   * @param {string} parameters.blobContainerSasUri SAS Uri of the logging blob
   * container to which LogAnalytics Api writes output logs to.
   *
   * @param {date} parameters.fromTime From time of the query
   *
   * @param {date} parameters.toTime To time of the query
   *
   * @param {boolean} [parameters.groupByThrottlePolicy] Group query result by
   * Throttle Policy applied.
   *
   * @param {boolean} [parameters.groupByOperationName] Group query result by  by
   * Operation Name.
   *
   * @param {boolean} [parameters.groupByResourceName] Group query result by
   * Resource Name.
   *
   * @param {string} location The location upon which virtual-machine-sizes is
   * queried.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {LogAnalyticsOperationResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link LogAnalyticsOperationResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginExportThrottledRequests(parameters, location, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginExportThrottledRequests(parameters, location, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginExportThrottledRequests(parameters, location, options, optionalCallback);
    }
  }

}

module.exports = LogAnalytics;
