/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 *
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

const msRest = require('ms-rest');
const msRestAzure = require('ms-rest-azure');
const WebResource = msRest.WebResource;


/**
 * Create or update a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set to create or
 * update.
 *
 * @param {object} parameters The scale set object.
 *
 * @param {object} [parameters.sku] The virtual machine scale set sku.
 *
 * @param {string} [parameters.sku.name] The sku name.
 *
 * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
 * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
 * /><br /> **Basic**
 *
 * @param {number} [parameters.sku.capacity] Specifies the number of virtual
 * machines in the scale set.
 *
 * @param {object} [parameters.plan] Specifies information about the
 * marketplace image used to create the virtual machine. This element is only
 * used for marketplace images. Before you can use a marketplace image from an
 * API, you must enable the image for programmatic use.  In the Azure portal,
 * find the marketplace image that you want to use and then click **Want to
 * deploy programmatically, Get Started ->**. Enter any required information
 * and then click **Save**.
 *
 * @param {string} [parameters.plan.name] The plan ID.
 *
 * @param {string} [parameters.plan.publisher] The publisher ID.
 *
 * @param {string} [parameters.plan.product] Specifies the product of the image
 * from the marketplace. This is the same value as Offer under the
 * imageReference element.
 *
 * @param {string} [parameters.plan.promotionCode] The promotion code.
 *
 * @param {object} [parameters.upgradePolicy] The upgrade policy.
 *
 * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
 * upgrade to virtual machines in the scale set.<br /><br /> Possible values
 * are:<br /><br /> **Manual** - You  control the application of updates to
 * virtual machines in the scale set. You do this by using the manualUpgrade
 * action.<br /><br /> **Automatic** - All virtual machines in the scale set
 * are  automatically updated at the same time. Possible values include:
 * 'Automatic', 'Manual', 'Rolling'
 *
 * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
 * configuration parameters used while performing a rolling upgrade.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
 * maximum percent of total virtual machine instances that will be upgraded
 * simultaneously by the rolling upgrade in one batch. As this is a maximum,
 * unhealthy instances in previous or future batches can cause the percentage
 * of instances in a batch to decrease to ensure higher reliability. The
 * default value for this parameter is 20%.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
 * The maximum percentage of the total virtual machine instances in the scale
 * set that can be simultaneously unhealthy, either as a result of being
 * upgraded, or by being found in an unhealthy state by the virtual machine
 * health checks before the rolling upgrade aborts. This constraint will be
 * checked prior to starting any batch. The default value for this parameter is
 * 20%.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
 * The maximum percentage of upgraded virtual machine instances that can be
 * found to be in an unhealthy state. This check will happen after each batch
 * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
 * The default value for this parameter is 20%.
 *
 * @param {string}
 * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
 * wait time between completing the update for all virtual machines in one
 * batch and starting the next batch. The time duration should be specified in
 * ISO 8601 format. The default value is 0 seconds (PT0S).
 *
 * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
 * Configuration parameters used for performing automatic OS Upgrade.
 *
 * @param {boolean}
 * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
 * Whether OS upgrades should automatically be applied to scale set instances
 * in a rolling fashion when a newer version of the image becomes available.
 * Default value is false.
 *
 * @param {boolean}
 * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
 * Whether OS image rollback feature should be disabled. Default value is
 * false.
 *
 * @param {object} [parameters.virtualMachineProfile] The virtual machine
 * profile.
 *
 * @param {object} [parameters.virtualMachineProfile.osProfile] Specifies the
 * operating system settings for the virtual machines in the scale set.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.osProfile.computerNamePrefix] Specifies
 * the computer name prefix for all of the virtual machines in the scale set.
 * Computer name prefixes must be 1 to 15 characters long.
 *
 * @param {string} [parameters.virtualMachineProfile.osProfile.adminUsername]
 * Specifies the name of the administrator account. <br><br> **Windows-only
 * restriction:** Cannot end in "." <br><br> **Disallowed values:**
 * "administrator", "admin", "user", "user1", "test", "user2", "test1",
 * "user3", "admin1", "1", "123", "a", "actuser", "adm", "admin2", "aspnet",
 * "backup", "console", "david", "guest", "john", "owner", "root", "server",
 * "sql", "support", "support_388945a0", "sys", "test2", "test3", "user4",
 * "user5". <br><br> **Minimum-length (Linux):** 1  character <br><br>
 * **Max-length (Linux):** 64 characters <br><br> **Max-length (Windows):** 20
 * characters  <br><br><li> For root access to the Linux VM, see [Using root
 * privileges on Linux virtual machines in
 * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-use-root-privileges?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)<br><li>
 * For a list of built-in system users on Linux that should not be used in this
 * field, see [Selecting User Names for Linux on
 * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-usernames?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
 *
 * @param {string} [parameters.virtualMachineProfile.osProfile.adminPassword]
 * Specifies the password of the administrator account. <br><br>
 * **Minimum-length (Windows):** 8 characters <br><br> **Minimum-length
 * (Linux):** 6 characters <br><br> **Max-length (Windows):** 123 characters
 * <br><br> **Max-length (Linux):** 72 characters <br><br> **Complexity
 * requirements:** 3 out of 4 conditions below need to be fulfilled <br> Has
 * lower characters <br>Has upper characters <br> Has a digit <br> Has a
 * special character (Regex match [\W_]) <br><br> **Disallowed values:**
 * "abc@123", "P@$$w0rd", "P@ssw0rd", "P@ssword123", "Pa$$word", "pass@word1",
 * "Password!", "Password1", "Password22", "iloveyou!" <br><br> For resetting
 * the password, see [How to reset the Remote Desktop service or its login
 * password in a Windows
 * VM](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-reset-rdp?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
 * <br><br> For resetting root password, see [Manage users, SSH, and check or
 * repair disks on Azure Linux VMs using the VMAccess
 * Extension](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-vmaccess-extension?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json#reset-root-password)
 *
 * @param {string} [parameters.virtualMachineProfile.osProfile.customData]
 * Specifies a base-64 encoded string of custom data. The base-64 encoded
 * string is decoded to a binary array that is saved as a file on the Virtual
 * Machine. The maximum length of the binary array is 65535 bytes. <br><br> For
 * using cloud-init for your VM, see [Using cloud-init to customize a Linux VM
 * during
 * creation](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-cloud-init?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] Specifies
 * Windows operating system settings on the virtual machine.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
 * Indicates whether virtual machine agent should be provisioned on the virtual
 * machine. <br><br> When this property is not specified in the request body,
 * default behavior is to set it to true.  This will ensure that VM Agent is
 * installed on the VM so that extensions can be added to the VM later.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
 * Indicates whether virtual machine is enabled for automatic updates.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
 * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
 * Specifies additional base-64 encoded XML formatted information that can be
 * included in the Unattend.xml file, which is used by Windows Setup.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
 * Specifies the Windows Remote Management listeners. This enables remote
 * Windows PowerShell.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
 * The list of Windows Remote Management listeners
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] Specifies
 * the Linux operating system settings on the virtual machine. <br><br>For a
 * list of supported Linux distributions, see [Linux on Azure-Endorsed
 * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-endorsed-distros?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
 * <br><br> For running non-endorsed distributions, see [Information for
 * Non-Endorsed
 * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-create-upload-generic?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json).
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
 * Specifies whether password authentication should be disabled.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
 * Specifies the ssh key configuration for a Linux OS.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
 * The list of SSH public keys used to authenticate with linux based VMs.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
 * Indicates whether virtual machine agent should be provisioned on the virtual
 * machine. <br><br> When this property is not specified in the request body,
 * default behavior is to set it to true.  This will ensure that VM Agent is
 * installed on the VM so that extensions can be added to the VM later.
 *
 * @param {array} [parameters.virtualMachineProfile.osProfile.secrets]
 * Specifies set of certificates that should be installed onto the virtual
 * machines in the scale set.
 *
 * @param {object} [parameters.virtualMachineProfile.storageProfile] Specifies
 * the storage settings for the virtual machine disks.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.imageReference] Specifies
 * information about the image to use. You can specify information about
 * platform images, marketplace images, or virtual machine images. This element
 * is required when you want to use a platform image, marketplace image, or
 * virtual machine image, but is not used in other creation operations.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
 * The image publisher.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
 * Specifies the offer of the platform image or marketplace image used to
 * create the virtual machine.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
 * image SKU.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
 * Specifies the version of the platform image or marketplace image used to
 * create the virtual machine. The allowed formats are Major.Minor.Build or
 * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
 * use the latest version of an image available at deploy time. Even if you use
 * 'latest', the VM image will not automatically update after deploy time even
 * if a new version becomes available.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
 * Id
 *
 * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk]
 * Specifies information about the operating system disk used by the virtual
 * machines in the scale set. <br><br> For more information about disks, see
 * [About disks and VHDs for Azure virtual
 * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.name] The disk name.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] Specifies
 * the caching requirements. <br><br> Possible values are: <br><br> **None**
 * <br><br> **ReadOnly** <br><br> **ReadWrite** <br><br> Default: **None for
 * Standard storage. ReadOnly for Premium storage**. Possible values include:
 * 'None', 'ReadOnly', 'ReadWrite'
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
 * Specifies whether writeAccelerator should be enabled or disabled on the
 * disk.
 *
 * @param {string}
 * parameters.virtualMachineProfile.storageProfile.osDisk.createOption
 * Specifies how the virtual machines in the scale set should be
 * created.<br><br> The only allowed value is: **FromImage** \u2013 This value
 * is used when you are using an image to create the virtual machine. If you
 * are using a platform image, you also use the imageReference element
 * described above. If you are using a marketplace image, you  also use the
 * plan element previously described. Possible values include: 'FromImage',
 * 'Empty', 'Attach'
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings]
 * Specifies the differencing Disk Settings for the operating system disk used
 * by the virtual machine scale set.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings.option]
 * Specifies the differencing disk settings for operating system disk. Possible
 * values include: 'Local'
 *
 * @param {number}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
 * Specifies the size of the operating system disk in gigabytes. This element
 * can be used to overwrite the size of the disk in a virtual machine image.
 * <br><br> This value cannot be larger than 1023 GB
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.osType] This
 * property allows you to specify the type of the OS that is included in the
 * disk if creating a VM from user-image or a specialized VHD. <br><br>
 * Possible values are: <br><br> **Windows** <br><br> **Linux**. Possible
 * values include: 'Windows', 'Linux'
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.image] Specifies
 * information about the unmanaged user image to base the scale set on.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
 * the virtual hard disk's uri.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers]
 * Specifies the container urls that are used to store operating system disks
 * for the scale set.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
 * managed disk parameters.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
 * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
 * can only be used with data disks, it cannot be used with OS Disk. Possible
 * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
 * 'UltraSSD_LRS'
 *
 * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
 * Specifies the parameters that are used to add data disks to the virtual
 * machines in the scale set. <br><br> For more information about disks, see
 * [About disks and VHDs for Azure virtual
 * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
 *
 * @param {object} [parameters.virtualMachineProfile.additionalCapabilities]
 * Specifies additional capabilities enabled or disabled on the virtual machine
 * in the scale set. For instance: whether the virtual machine has the
 * capability to support attaching managed data disks with UltraSSD_LRS storage
 * account type.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.additionalCapabilities.ultraSSDEnabled]
 * The flag that enables or disables a capability to have one or more managed
 * data disks with UltraSSD_LRS storage account type on the VM or VMSS. Managed
 * disks with storage account type UltraSSD_LRS can be added to a virtual
 * machine or virtual machine scale set only if this property is enabled.
 *
 * @param {object} [parameters.virtualMachineProfile.networkProfile] Specifies
 * properties of the network interfaces of the virtual machines in the scale
 * set.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.networkProfile.healthProbe] A reference to
 * a load balancer probe used to determine the health of an instance in the
 * virtual machine scale set. The reference will be in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/loadBalancers/{loadBalancerName}/probes/{probeName}'.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.networkProfile.healthProbe.id] The ARM
 * resource id in the form of
 * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
 *
 * @param {array}
 * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
 * The list of network configurations.
 *
 * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile]
 * Specifies the boot diagnostic settings state. <br><br>Minimum api-version:
 * 2015-06-15.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
 * Diagnostics is a debugging feature which allows you to view Console Output
 * and Screenshot to diagnose VM status. <br><br> You can easily view the
 * output of your console log. <br><br> Azure also enables you to see a
 * screenshot of the VM from the hypervisor.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
 * Whether boot diagnostics should be enabled on the Virtual Machine.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
 * Uri of the storage account to use for placing the console output and
 * screenshot.
 *
 * @param {object} [parameters.virtualMachineProfile.extensionProfile]
 * Specifies a collection of settings for extensions installed on virtual
 * machines in the scale set.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
 * machine scale set child extension resources.
 *
 * @param {string} [parameters.virtualMachineProfile.licenseType] Specifies
 * that the image or disk that is being used was licensed on-premises. This
 * element is only used for images that contain the Windows Server operating
 * system. <br><br> Possible values are: <br><br> Windows_Client <br><br>
 * Windows_Server <br><br> If this element is included in a request for an
 * update, the value must match the initial value. This value cannot be
 * updated. <br><br> For more information, see [Azure Hybrid Use Benefit for
 * Windows
 * Server](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-hybrid-use-benefit-licensing?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
 * <br><br> Minimum api-version: 2015-06-15
 *
 * @param {string} [parameters.virtualMachineProfile.priority] Specifies the
 * priority for the virtual machines in the scale set. <br><br>Minimum
 * api-version: 2017-10-30-preview. Possible values include: 'Regular', 'Low'
 *
 * @param {string} [parameters.virtualMachineProfile.evictionPolicy] Specifies
 * the eviction policy for virtual machines in a low priority scale set.
 * <br><br>Minimum api-version: 2017-10-30-preview. Possible values include:
 * 'Deallocate', 'Delete'
 *
 * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
 * Machine Scale Set should be overprovisioned.
 *
 * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
 * scale set to a single placement group, of max size 100 virtual machines.
 *
 * @param {boolean} [parameters.zoneBalance] Whether to force stictly even
 * Virtual Machine distribution cross x-zones in case there is zone outage.
 *
 * @param {number} [parameters.platformFaultDomainCount] Fault Domain count for
 * each placement group.
 *
 * @param {object} [parameters.identity] The identity of the virtual machine
 * scale set, if configured.
 *
 * @param {string} [parameters.identity.type] The type of identity used for the
 * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
 * both an implicitly created identity and a set of user assigned identities.
 * The type 'None' will remove any identities from the virtual machine scale
 * set. Possible values include: 'SystemAssigned', 'UserAssigned',
 * 'SystemAssigned, UserAssigned', 'None'
 *
 * @param {object} [parameters.identity.userAssignedIdentities] The list of
 * user identities associated with the virtual machine scale set. The user
 * identity dictionary key references will be ARM resource ids in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
 *
 * @param {array} [parameters.zones] The virtual machine scale set zones.
 *
 * @param {string} parameters.location Resource location
 *
 * @param {object} [parameters.tags] Resource tags
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSet} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _createOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginCreateOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSet']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Update a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set to create or
 * update.
 *
 * @param {object} parameters The scale set object.
 *
 * @param {object} [parameters.sku] The virtual machine scale set sku.
 *
 * @param {string} [parameters.sku.name] The sku name.
 *
 * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
 * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
 * /><br /> **Basic**
 *
 * @param {number} [parameters.sku.capacity] Specifies the number of virtual
 * machines in the scale set.
 *
 * @param {object} [parameters.plan] The purchase plan when deploying a virtual
 * machine scale set from VM Marketplace images.
 *
 * @param {string} [parameters.plan.name] The plan ID.
 *
 * @param {string} [parameters.plan.publisher] The publisher ID.
 *
 * @param {string} [parameters.plan.product] Specifies the product of the image
 * from the marketplace. This is the same value as Offer under the
 * imageReference element.
 *
 * @param {string} [parameters.plan.promotionCode] The promotion code.
 *
 * @param {object} [parameters.upgradePolicy] The upgrade policy.
 *
 * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
 * upgrade to virtual machines in the scale set.<br /><br /> Possible values
 * are:<br /><br /> **Manual** - You  control the application of updates to
 * virtual machines in the scale set. You do this by using the manualUpgrade
 * action.<br /><br /> **Automatic** - All virtual machines in the scale set
 * are  automatically updated at the same time. Possible values include:
 * 'Automatic', 'Manual', 'Rolling'
 *
 * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
 * configuration parameters used while performing a rolling upgrade.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
 * maximum percent of total virtual machine instances that will be upgraded
 * simultaneously by the rolling upgrade in one batch. As this is a maximum,
 * unhealthy instances in previous or future batches can cause the percentage
 * of instances in a batch to decrease to ensure higher reliability. The
 * default value for this parameter is 20%.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
 * The maximum percentage of the total virtual machine instances in the scale
 * set that can be simultaneously unhealthy, either as a result of being
 * upgraded, or by being found in an unhealthy state by the virtual machine
 * health checks before the rolling upgrade aborts. This constraint will be
 * checked prior to starting any batch. The default value for this parameter is
 * 20%.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
 * The maximum percentage of upgraded virtual machine instances that can be
 * found to be in an unhealthy state. This check will happen after each batch
 * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
 * The default value for this parameter is 20%.
 *
 * @param {string}
 * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
 * wait time between completing the update for all virtual machines in one
 * batch and starting the next batch. The time duration should be specified in
 * ISO 8601 format. The default value is 0 seconds (PT0S).
 *
 * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
 * Configuration parameters used for performing automatic OS Upgrade.
 *
 * @param {boolean}
 * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
 * Whether OS upgrades should automatically be applied to scale set instances
 * in a rolling fashion when a newer version of the image becomes available.
 * Default value is false.
 *
 * @param {boolean}
 * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
 * Whether OS image rollback feature should be disabled. Default value is
 * false.
 *
 * @param {object} [parameters.virtualMachineProfile] The virtual machine
 * profile.
 *
 * @param {object} [parameters.virtualMachineProfile.osProfile] The virtual
 * machine scale set OS profile.
 *
 * @param {string} [parameters.virtualMachineProfile.osProfile.customData] A
 * base-64 encoded string of custom data.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] The
 * Windows Configuration of the OS profile.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
 * Indicates whether virtual machine agent should be provisioned on the virtual
 * machine. <br><br> When this property is not specified in the request body,
 * default behavior is to set it to true.  This will ensure that VM Agent is
 * installed on the VM so that extensions can be added to the VM later.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
 * Indicates whether virtual machine is enabled for automatic updates.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
 * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
 * Specifies additional base-64 encoded XML formatted information that can be
 * included in the Unattend.xml file, which is used by Windows Setup.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
 * Specifies the Windows Remote Management listeners. This enables remote
 * Windows PowerShell.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
 * The list of Windows Remote Management listeners
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] The Linux
 * Configuration of the OS profile.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
 * Specifies whether password authentication should be disabled.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
 * Specifies the ssh key configuration for a Linux OS.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
 * The list of SSH public keys used to authenticate with linux based VMs.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
 * Indicates whether virtual machine agent should be provisioned on the virtual
 * machine. <br><br> When this property is not specified in the request body,
 * default behavior is to set it to true.  This will ensure that VM Agent is
 * installed on the VM so that extensions can be added to the VM later.
 *
 * @param {array} [parameters.virtualMachineProfile.osProfile.secrets] The List
 * of certificates for addition to the VM.
 *
 * @param {object} [parameters.virtualMachineProfile.storageProfile] The
 * virtual machine scale set storage profile.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.imageReference] The image
 * reference.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
 * The image publisher.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
 * Specifies the offer of the platform image or marketplace image used to
 * create the virtual machine.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
 * image SKU.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
 * Specifies the version of the platform image or marketplace image used to
 * create the virtual machine. The allowed formats are Major.Minor.Build or
 * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
 * use the latest version of an image available at deploy time. Even if you use
 * 'latest', the VM image will not automatically update after deploy time even
 * if a new version becomes available.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
 * Id
 *
 * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk] The
 * OS disk.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] The caching
 * type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
 * Specifies whether writeAccelerator should be enabled or disabled on the
 * disk.
 *
 * @param {number}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
 * Specifies the size of the operating system disk in gigabytes. This element
 * can be used to overwrite the size of the disk in a virtual machine image.
 * <br><br> This value cannot be larger than 1023 GB
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.image] The Source
 * User Image VirtualHardDisk. This VirtualHardDisk will be copied before using
 * it to attach to the Virtual Machine. If SourceImage is provided, the
 * destination VirtualHardDisk should not exist.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
 * the virtual hard disk's uri.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers] The
 * list of virtual hard disk container uris.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
 * managed disk parameters.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
 * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
 * can only be used with data disks, it cannot be used with OS Disk. Possible
 * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
 * 'UltraSSD_LRS'
 *
 * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
 * The data disks.
 *
 * @param {object} [parameters.virtualMachineProfile.networkProfile] The
 * virtual machine scale set network profile.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
 * The list of network configurations.
 *
 * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile] The
 * virtual machine scale set diagnostics profile.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
 * Diagnostics is a debugging feature which allows you to view Console Output
 * and Screenshot to diagnose VM status. <br><br> You can easily view the
 * output of your console log. <br><br> Azure also enables you to see a
 * screenshot of the VM from the hypervisor.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
 * Whether boot diagnostics should be enabled on the Virtual Machine.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
 * Uri of the storage account to use for placing the console output and
 * screenshot.
 *
 * @param {object} [parameters.virtualMachineProfile.extensionProfile] The
 * virtual machine scale set extension profile.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
 * machine scale set child extension resources.
 *
 * @param {string} [parameters.virtualMachineProfile.licenseType] The license
 * type, which is for bring your own license scenario.
 *
 * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
 * Machine Scale Set should be overprovisioned.
 *
 * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
 * scale set to a single placement group, of max size 100 virtual machines.
 *
 * @param {object} [parameters.identity] The identity of the virtual machine
 * scale set, if configured.
 *
 * @param {string} [parameters.identity.type] The type of identity used for the
 * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
 * both an implicitly created identity and a set of user assigned identities.
 * The type 'None' will remove any identities from the virtual machine scale
 * set. Possible values include: 'SystemAssigned', 'UserAssigned',
 * 'SystemAssigned, UserAssigned', 'None'
 *
 * @param {object} [parameters.identity.userAssignedIdentities] The list of
 * user identities associated with the virtual machine scale set. The user
 * identity dictionary key references will be ARM resource ids in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
 *
 * @param {object} [parameters.tags] Resource tags
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSet} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _update(resourceGroupName, vmScaleSetName, parameters, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginUpdate(resourceGroupName, vmScaleSetName, parameters, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSet']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Deletes a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _deleteMethod(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginDeleteMethod(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}

/**
 * Display information about a virtual machine scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSet} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _get(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSet']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}


/**
 * Deallocates specific virtual machines in a VM scale set. Shuts down the
 * virtual machines and releases the compute resources. You are not billed for
 * the compute resources that this virtual machine scale set deallocates.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _deallocate(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginDeallocate(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Deletes virtual machines in a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {array} instanceIds The virtual machine scale set instance ids.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _deleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginDeleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}

/**
 * Gets the status of a VM scale set instance.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetInstanceView} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _getInstanceView(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/instanceView';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetInstanceView']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets a list of all VM scale sets under a resource group.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetListResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _list(resourceGroupName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetListResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets a list of all VM Scale Sets in the subscription, regardless of the
 * associated resource group. Use nextLink property in the response to get the
 * next page of VM Scale Sets. Do this till nextLink is null to fetch all the
 * VM Scale Sets.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetListWithLinkResult}
 *                      for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _listAll(options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/providers/Microsoft.Compute/virtualMachineScaleSets';
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetListWithLinkResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets a list of SKUs available for your VM scale set, including the minimum
 * and maximum VM instances allowed for each SKU.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetListSkusResult} for
 *                      more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _listSkus(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/skus';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetListSkusResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets list of OS upgrades on a VM scale set instance.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetListOSUpgradeHistory}
 *                      for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _getOSUpgradeHistory(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/osUpgradeHistory';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetListOSUpgradeHistory']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}


/**
 * Power off (stop) one or more virtual machines in a VM scale set. Note that
 * resources are still attached and you are getting charged for the resources.
 * Instead, use deallocate to release resources and avoid charges.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _powerOff(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginPowerOff(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Restarts one or more virtual machines in a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _restart(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginRestart(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Starts one or more virtual machines in a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _start(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginStart(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Redeploy one or more virtual machines in a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _redeploy(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginRedeploy(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Perform maintenance on one or more virtual machines in a VM scale set.
 * Operation on instances which are not eligible for perform maintenance will
 * be failed. Please refer to best practices for more details:
 * https://docs.microsoft.com/en-us/azure/virtual-machine-scale-sets/virtual-machine-scale-sets-maintenance-notifications
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _performMaintenance(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginPerformMaintenance(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Upgrades one or more virtual machines to the latest SKU set in the VM scale
 * set model.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {array} instanceIds The virtual machine scale set instance ids.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _updateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginUpdateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Reimages (upgrade the operating system) one or more virtual machines in a VM
 * scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _reimage(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginReimage(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}


/**
 * Reimages all the disks ( including data disks ) in the virtual machines in a
 * VM scale set. This operation is only supported for managed disks.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _reimageAll(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }

  if (!callback) {
    throw new Error('callback cannot be null.');
  }

  // Send request
  this.beginReimageAll(resourceGroupName, vmScaleSetName, options, (err, parsedResult, httpRequest, response) => {
    if (err) return callback(err);

    let initialResult = new msRest.HttpOperationResponse();
    initialResult.request = httpRequest;
    initialResult.response = response;
    initialResult.body = response.body;
    client.getLongRunningOperationResult(initialResult, options, (err, pollingResult) => {
      if (err) return callback(err);

      // Create Result
      let result = null;

      httpRequest = pollingResult.request;
      response = pollingResult.response;
      let responseBody = pollingResult.body;
      if (responseBody === '') responseBody = null;

      // Deserialize Response

      return callback(null, result, httpRequest, response);
    });
  });
}

/**
 * Manual platform update domain walk to update virtual machines in a service
 * fabric virtual machine scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {number} platformUpdateDomain The platform update domain for which a
 * manual recovery walk is requested
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link RecoveryWalkResponse} for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _forceRecoveryServiceFabricPlatformUpdateDomainWalk(resourceGroupName, vmScaleSetName, platformUpdateDomain, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (platformUpdateDomain === null || platformUpdateDomain === undefined || typeof platformUpdateDomain !== 'number') {
      throw new Error('platformUpdateDomain cannot be null or undefined and it must be of type number.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/forceRecoveryServiceFabricPlatformUpdateDomainWalk';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  queryParameters.push('platformUpdateDomain=' + encodeURIComponent(platformUpdateDomain.toString()));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['RecoveryWalkResponse']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Create or update a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set to create or
 * update.
 *
 * @param {object} parameters The scale set object.
 *
 * @param {object} [parameters.sku] The virtual machine scale set sku.
 *
 * @param {string} [parameters.sku.name] The sku name.
 *
 * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
 * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
 * /><br /> **Basic**
 *
 * @param {number} [parameters.sku.capacity] Specifies the number of virtual
 * machines in the scale set.
 *
 * @param {object} [parameters.plan] Specifies information about the
 * marketplace image used to create the virtual machine. This element is only
 * used for marketplace images. Before you can use a marketplace image from an
 * API, you must enable the image for programmatic use.  In the Azure portal,
 * find the marketplace image that you want to use and then click **Want to
 * deploy programmatically, Get Started ->**. Enter any required information
 * and then click **Save**.
 *
 * @param {string} [parameters.plan.name] The plan ID.
 *
 * @param {string} [parameters.plan.publisher] The publisher ID.
 *
 * @param {string} [parameters.plan.product] Specifies the product of the image
 * from the marketplace. This is the same value as Offer under the
 * imageReference element.
 *
 * @param {string} [parameters.plan.promotionCode] The promotion code.
 *
 * @param {object} [parameters.upgradePolicy] The upgrade policy.
 *
 * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
 * upgrade to virtual machines in the scale set.<br /><br /> Possible values
 * are:<br /><br /> **Manual** - You  control the application of updates to
 * virtual machines in the scale set. You do this by using the manualUpgrade
 * action.<br /><br /> **Automatic** - All virtual machines in the scale set
 * are  automatically updated at the same time. Possible values include:
 * 'Automatic', 'Manual', 'Rolling'
 *
 * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
 * configuration parameters used while performing a rolling upgrade.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
 * maximum percent of total virtual machine instances that will be upgraded
 * simultaneously by the rolling upgrade in one batch. As this is a maximum,
 * unhealthy instances in previous or future batches can cause the percentage
 * of instances in a batch to decrease to ensure higher reliability. The
 * default value for this parameter is 20%.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
 * The maximum percentage of the total virtual machine instances in the scale
 * set that can be simultaneously unhealthy, either as a result of being
 * upgraded, or by being found in an unhealthy state by the virtual machine
 * health checks before the rolling upgrade aborts. This constraint will be
 * checked prior to starting any batch. The default value for this parameter is
 * 20%.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
 * The maximum percentage of upgraded virtual machine instances that can be
 * found to be in an unhealthy state. This check will happen after each batch
 * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
 * The default value for this parameter is 20%.
 *
 * @param {string}
 * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
 * wait time between completing the update for all virtual machines in one
 * batch and starting the next batch. The time duration should be specified in
 * ISO 8601 format. The default value is 0 seconds (PT0S).
 *
 * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
 * Configuration parameters used for performing automatic OS Upgrade.
 *
 * @param {boolean}
 * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
 * Whether OS upgrades should automatically be applied to scale set instances
 * in a rolling fashion when a newer version of the image becomes available.
 * Default value is false.
 *
 * @param {boolean}
 * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
 * Whether OS image rollback feature should be disabled. Default value is
 * false.
 *
 * @param {object} [parameters.virtualMachineProfile] The virtual machine
 * profile.
 *
 * @param {object} [parameters.virtualMachineProfile.osProfile] Specifies the
 * operating system settings for the virtual machines in the scale set.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.osProfile.computerNamePrefix] Specifies
 * the computer name prefix for all of the virtual machines in the scale set.
 * Computer name prefixes must be 1 to 15 characters long.
 *
 * @param {string} [parameters.virtualMachineProfile.osProfile.adminUsername]
 * Specifies the name of the administrator account. <br><br> **Windows-only
 * restriction:** Cannot end in "." <br><br> **Disallowed values:**
 * "administrator", "admin", "user", "user1", "test", "user2", "test1",
 * "user3", "admin1", "1", "123", "a", "actuser", "adm", "admin2", "aspnet",
 * "backup", "console", "david", "guest", "john", "owner", "root", "server",
 * "sql", "support", "support_388945a0", "sys", "test2", "test3", "user4",
 * "user5". <br><br> **Minimum-length (Linux):** 1  character <br><br>
 * **Max-length (Linux):** 64 characters <br><br> **Max-length (Windows):** 20
 * characters  <br><br><li> For root access to the Linux VM, see [Using root
 * privileges on Linux virtual machines in
 * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-use-root-privileges?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)<br><li>
 * For a list of built-in system users on Linux that should not be used in this
 * field, see [Selecting User Names for Linux on
 * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-usernames?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
 *
 * @param {string} [parameters.virtualMachineProfile.osProfile.adminPassword]
 * Specifies the password of the administrator account. <br><br>
 * **Minimum-length (Windows):** 8 characters <br><br> **Minimum-length
 * (Linux):** 6 characters <br><br> **Max-length (Windows):** 123 characters
 * <br><br> **Max-length (Linux):** 72 characters <br><br> **Complexity
 * requirements:** 3 out of 4 conditions below need to be fulfilled <br> Has
 * lower characters <br>Has upper characters <br> Has a digit <br> Has a
 * special character (Regex match [\W_]) <br><br> **Disallowed values:**
 * "abc@123", "P@$$w0rd", "P@ssw0rd", "P@ssword123", "Pa$$word", "pass@word1",
 * "Password!", "Password1", "Password22", "iloveyou!" <br><br> For resetting
 * the password, see [How to reset the Remote Desktop service or its login
 * password in a Windows
 * VM](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-reset-rdp?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
 * <br><br> For resetting root password, see [Manage users, SSH, and check or
 * repair disks on Azure Linux VMs using the VMAccess
 * Extension](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-vmaccess-extension?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json#reset-root-password)
 *
 * @param {string} [parameters.virtualMachineProfile.osProfile.customData]
 * Specifies a base-64 encoded string of custom data. The base-64 encoded
 * string is decoded to a binary array that is saved as a file on the Virtual
 * Machine. The maximum length of the binary array is 65535 bytes. <br><br> For
 * using cloud-init for your VM, see [Using cloud-init to customize a Linux VM
 * during
 * creation](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-cloud-init?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] Specifies
 * Windows operating system settings on the virtual machine.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
 * Indicates whether virtual machine agent should be provisioned on the virtual
 * machine. <br><br> When this property is not specified in the request body,
 * default behavior is to set it to true.  This will ensure that VM Agent is
 * installed on the VM so that extensions can be added to the VM later.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
 * Indicates whether virtual machine is enabled for automatic updates.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
 * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
 * Specifies additional base-64 encoded XML formatted information that can be
 * included in the Unattend.xml file, which is used by Windows Setup.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
 * Specifies the Windows Remote Management listeners. This enables remote
 * Windows PowerShell.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
 * The list of Windows Remote Management listeners
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] Specifies
 * the Linux operating system settings on the virtual machine. <br><br>For a
 * list of supported Linux distributions, see [Linux on Azure-Endorsed
 * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-endorsed-distros?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
 * <br><br> For running non-endorsed distributions, see [Information for
 * Non-Endorsed
 * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-create-upload-generic?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json).
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
 * Specifies whether password authentication should be disabled.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
 * Specifies the ssh key configuration for a Linux OS.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
 * The list of SSH public keys used to authenticate with linux based VMs.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
 * Indicates whether virtual machine agent should be provisioned on the virtual
 * machine. <br><br> When this property is not specified in the request body,
 * default behavior is to set it to true.  This will ensure that VM Agent is
 * installed on the VM so that extensions can be added to the VM later.
 *
 * @param {array} [parameters.virtualMachineProfile.osProfile.secrets]
 * Specifies set of certificates that should be installed onto the virtual
 * machines in the scale set.
 *
 * @param {object} [parameters.virtualMachineProfile.storageProfile] Specifies
 * the storage settings for the virtual machine disks.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.imageReference] Specifies
 * information about the image to use. You can specify information about
 * platform images, marketplace images, or virtual machine images. This element
 * is required when you want to use a platform image, marketplace image, or
 * virtual machine image, but is not used in other creation operations.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
 * The image publisher.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
 * Specifies the offer of the platform image or marketplace image used to
 * create the virtual machine.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
 * image SKU.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
 * Specifies the version of the platform image or marketplace image used to
 * create the virtual machine. The allowed formats are Major.Minor.Build or
 * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
 * use the latest version of an image available at deploy time. Even if you use
 * 'latest', the VM image will not automatically update after deploy time even
 * if a new version becomes available.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
 * Id
 *
 * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk]
 * Specifies information about the operating system disk used by the virtual
 * machines in the scale set. <br><br> For more information about disks, see
 * [About disks and VHDs for Azure virtual
 * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.name] The disk name.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] Specifies
 * the caching requirements. <br><br> Possible values are: <br><br> **None**
 * <br><br> **ReadOnly** <br><br> **ReadWrite** <br><br> Default: **None for
 * Standard storage. ReadOnly for Premium storage**. Possible values include:
 * 'None', 'ReadOnly', 'ReadWrite'
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
 * Specifies whether writeAccelerator should be enabled or disabled on the
 * disk.
 *
 * @param {string}
 * parameters.virtualMachineProfile.storageProfile.osDisk.createOption
 * Specifies how the virtual machines in the scale set should be
 * created.<br><br> The only allowed value is: **FromImage** \u2013 This value
 * is used when you are using an image to create the virtual machine. If you
 * are using a platform image, you also use the imageReference element
 * described above. If you are using a marketplace image, you  also use the
 * plan element previously described. Possible values include: 'FromImage',
 * 'Empty', 'Attach'
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings]
 * Specifies the differencing Disk Settings for the operating system disk used
 * by the virtual machine scale set.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings.option]
 * Specifies the differencing disk settings for operating system disk. Possible
 * values include: 'Local'
 *
 * @param {number}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
 * Specifies the size of the operating system disk in gigabytes. This element
 * can be used to overwrite the size of the disk in a virtual machine image.
 * <br><br> This value cannot be larger than 1023 GB
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.osType] This
 * property allows you to specify the type of the OS that is included in the
 * disk if creating a VM from user-image or a specialized VHD. <br><br>
 * Possible values are: <br><br> **Windows** <br><br> **Linux**. Possible
 * values include: 'Windows', 'Linux'
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.image] Specifies
 * information about the unmanaged user image to base the scale set on.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
 * the virtual hard disk's uri.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers]
 * Specifies the container urls that are used to store operating system disks
 * for the scale set.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
 * managed disk parameters.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
 * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
 * can only be used with data disks, it cannot be used with OS Disk. Possible
 * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
 * 'UltraSSD_LRS'
 *
 * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
 * Specifies the parameters that are used to add data disks to the virtual
 * machines in the scale set. <br><br> For more information about disks, see
 * [About disks and VHDs for Azure virtual
 * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
 *
 * @param {object} [parameters.virtualMachineProfile.additionalCapabilities]
 * Specifies additional capabilities enabled or disabled on the virtual machine
 * in the scale set. For instance: whether the virtual machine has the
 * capability to support attaching managed data disks with UltraSSD_LRS storage
 * account type.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.additionalCapabilities.ultraSSDEnabled]
 * The flag that enables or disables a capability to have one or more managed
 * data disks with UltraSSD_LRS storage account type on the VM or VMSS. Managed
 * disks with storage account type UltraSSD_LRS can be added to a virtual
 * machine or virtual machine scale set only if this property is enabled.
 *
 * @param {object} [parameters.virtualMachineProfile.networkProfile] Specifies
 * properties of the network interfaces of the virtual machines in the scale
 * set.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.networkProfile.healthProbe] A reference to
 * a load balancer probe used to determine the health of an instance in the
 * virtual machine scale set. The reference will be in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/loadBalancers/{loadBalancerName}/probes/{probeName}'.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.networkProfile.healthProbe.id] The ARM
 * resource id in the form of
 * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
 *
 * @param {array}
 * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
 * The list of network configurations.
 *
 * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile]
 * Specifies the boot diagnostic settings state. <br><br>Minimum api-version:
 * 2015-06-15.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
 * Diagnostics is a debugging feature which allows you to view Console Output
 * and Screenshot to diagnose VM status. <br><br> You can easily view the
 * output of your console log. <br><br> Azure also enables you to see a
 * screenshot of the VM from the hypervisor.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
 * Whether boot diagnostics should be enabled on the Virtual Machine.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
 * Uri of the storage account to use for placing the console output and
 * screenshot.
 *
 * @param {object} [parameters.virtualMachineProfile.extensionProfile]
 * Specifies a collection of settings for extensions installed on virtual
 * machines in the scale set.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
 * machine scale set child extension resources.
 *
 * @param {string} [parameters.virtualMachineProfile.licenseType] Specifies
 * that the image or disk that is being used was licensed on-premises. This
 * element is only used for images that contain the Windows Server operating
 * system. <br><br> Possible values are: <br><br> Windows_Client <br><br>
 * Windows_Server <br><br> If this element is included in a request for an
 * update, the value must match the initial value. This value cannot be
 * updated. <br><br> For more information, see [Azure Hybrid Use Benefit for
 * Windows
 * Server](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-hybrid-use-benefit-licensing?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
 * <br><br> Minimum api-version: 2015-06-15
 *
 * @param {string} [parameters.virtualMachineProfile.priority] Specifies the
 * priority for the virtual machines in the scale set. <br><br>Minimum
 * api-version: 2017-10-30-preview. Possible values include: 'Regular', 'Low'
 *
 * @param {string} [parameters.virtualMachineProfile.evictionPolicy] Specifies
 * the eviction policy for virtual machines in a low priority scale set.
 * <br><br>Minimum api-version: 2017-10-30-preview. Possible values include:
 * 'Deallocate', 'Delete'
 *
 * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
 * Machine Scale Set should be overprovisioned.
 *
 * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
 * scale set to a single placement group, of max size 100 virtual machines.
 *
 * @param {boolean} [parameters.zoneBalance] Whether to force stictly even
 * Virtual Machine distribution cross x-zones in case there is zone outage.
 *
 * @param {number} [parameters.platformFaultDomainCount] Fault Domain count for
 * each placement group.
 *
 * @param {object} [parameters.identity] The identity of the virtual machine
 * scale set, if configured.
 *
 * @param {string} [parameters.identity.type] The type of identity used for the
 * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
 * both an implicitly created identity and a set of user assigned identities.
 * The type 'None' will remove any identities from the virtual machine scale
 * set. Possible values include: 'SystemAssigned', 'UserAssigned',
 * 'SystemAssigned, UserAssigned', 'None'
 *
 * @param {object} [parameters.identity.userAssignedIdentities] The list of
 * user identities associated with the virtual machine scale set. The user
 * identity dictionary key references will be ARM resource ids in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
 *
 * @param {array} [parameters.zones] The virtual machine scale set zones.
 *
 * @param {string} parameters.location Resource location
 *
 * @param {object} [parameters.tags] Resource tags
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSet} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginCreateOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (parameters === null || parameters === undefined) {
      throw new Error('parameters cannot be null or undefined.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'PUT';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (parameters !== null && parameters !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSet']().mapper();
      requestModel = client.serialize(requestModelMapper, parameters, 'parameters');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(parameters, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 201) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSet']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }
    // Deserialize Response
    if (statusCode === 201) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSet']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError1 = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError1.request = msRest.stripRequest(httpRequest);
        deserializationError1.response = msRest.stripResponse(response);
        return callback(deserializationError1);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Update a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set to create or
 * update.
 *
 * @param {object} parameters The scale set object.
 *
 * @param {object} [parameters.sku] The virtual machine scale set sku.
 *
 * @param {string} [parameters.sku.name] The sku name.
 *
 * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
 * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
 * /><br /> **Basic**
 *
 * @param {number} [parameters.sku.capacity] Specifies the number of virtual
 * machines in the scale set.
 *
 * @param {object} [parameters.plan] The purchase plan when deploying a virtual
 * machine scale set from VM Marketplace images.
 *
 * @param {string} [parameters.plan.name] The plan ID.
 *
 * @param {string} [parameters.plan.publisher] The publisher ID.
 *
 * @param {string} [parameters.plan.product] Specifies the product of the image
 * from the marketplace. This is the same value as Offer under the
 * imageReference element.
 *
 * @param {string} [parameters.plan.promotionCode] The promotion code.
 *
 * @param {object} [parameters.upgradePolicy] The upgrade policy.
 *
 * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
 * upgrade to virtual machines in the scale set.<br /><br /> Possible values
 * are:<br /><br /> **Manual** - You  control the application of updates to
 * virtual machines in the scale set. You do this by using the manualUpgrade
 * action.<br /><br /> **Automatic** - All virtual machines in the scale set
 * are  automatically updated at the same time. Possible values include:
 * 'Automatic', 'Manual', 'Rolling'
 *
 * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
 * configuration parameters used while performing a rolling upgrade.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
 * maximum percent of total virtual machine instances that will be upgraded
 * simultaneously by the rolling upgrade in one batch. As this is a maximum,
 * unhealthy instances in previous or future batches can cause the percentage
 * of instances in a batch to decrease to ensure higher reliability. The
 * default value for this parameter is 20%.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
 * The maximum percentage of the total virtual machine instances in the scale
 * set that can be simultaneously unhealthy, either as a result of being
 * upgraded, or by being found in an unhealthy state by the virtual machine
 * health checks before the rolling upgrade aborts. This constraint will be
 * checked prior to starting any batch. The default value for this parameter is
 * 20%.
 *
 * @param {number}
 * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
 * The maximum percentage of upgraded virtual machine instances that can be
 * found to be in an unhealthy state. This check will happen after each batch
 * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
 * The default value for this parameter is 20%.
 *
 * @param {string}
 * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
 * wait time between completing the update for all virtual machines in one
 * batch and starting the next batch. The time duration should be specified in
 * ISO 8601 format. The default value is 0 seconds (PT0S).
 *
 * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
 * Configuration parameters used for performing automatic OS Upgrade.
 *
 * @param {boolean}
 * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
 * Whether OS upgrades should automatically be applied to scale set instances
 * in a rolling fashion when a newer version of the image becomes available.
 * Default value is false.
 *
 * @param {boolean}
 * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
 * Whether OS image rollback feature should be disabled. Default value is
 * false.
 *
 * @param {object} [parameters.virtualMachineProfile] The virtual machine
 * profile.
 *
 * @param {object} [parameters.virtualMachineProfile.osProfile] The virtual
 * machine scale set OS profile.
 *
 * @param {string} [parameters.virtualMachineProfile.osProfile.customData] A
 * base-64 encoded string of custom data.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] The
 * Windows Configuration of the OS profile.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
 * Indicates whether virtual machine agent should be provisioned on the virtual
 * machine. <br><br> When this property is not specified in the request body,
 * default behavior is to set it to true.  This will ensure that VM Agent is
 * installed on the VM so that extensions can be added to the VM later.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
 * Indicates whether virtual machine is enabled for automatic updates.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
 * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
 * Specifies additional base-64 encoded XML formatted information that can be
 * included in the Unattend.xml file, which is used by Windows Setup.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
 * Specifies the Windows Remote Management listeners. This enables remote
 * Windows PowerShell.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
 * The list of Windows Remote Management listeners
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] The Linux
 * Configuration of the OS profile.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
 * Specifies whether password authentication should be disabled.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
 * Specifies the ssh key configuration for a Linux OS.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
 * The list of SSH public keys used to authenticate with linux based VMs.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
 * Indicates whether virtual machine agent should be provisioned on the virtual
 * machine. <br><br> When this property is not specified in the request body,
 * default behavior is to set it to true.  This will ensure that VM Agent is
 * installed on the VM so that extensions can be added to the VM later.
 *
 * @param {array} [parameters.virtualMachineProfile.osProfile.secrets] The List
 * of certificates for addition to the VM.
 *
 * @param {object} [parameters.virtualMachineProfile.storageProfile] The
 * virtual machine scale set storage profile.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.imageReference] The image
 * reference.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
 * The image publisher.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
 * Specifies the offer of the platform image or marketplace image used to
 * create the virtual machine.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
 * image SKU.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
 * Specifies the version of the platform image or marketplace image used to
 * create the virtual machine. The allowed formats are Major.Minor.Build or
 * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
 * use the latest version of an image available at deploy time. Even if you use
 * 'latest', the VM image will not automatically update after deploy time even
 * if a new version becomes available.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
 * Id
 *
 * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk] The
 * OS disk.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] The caching
 * type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
 * Specifies whether writeAccelerator should be enabled or disabled on the
 * disk.
 *
 * @param {number}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
 * Specifies the size of the operating system disk in gigabytes. This element
 * can be used to overwrite the size of the disk in a virtual machine image.
 * <br><br> This value cannot be larger than 1023 GB
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.image] The Source
 * User Image VirtualHardDisk. This VirtualHardDisk will be copied before using
 * it to attach to the Virtual Machine. If SourceImage is provided, the
 * destination VirtualHardDisk should not exist.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
 * the virtual hard disk's uri.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers] The
 * list of virtual hard disk container uris.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
 * managed disk parameters.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
 * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
 * can only be used with data disks, it cannot be used with OS Disk. Possible
 * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
 * 'UltraSSD_LRS'
 *
 * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
 * The data disks.
 *
 * @param {object} [parameters.virtualMachineProfile.networkProfile] The
 * virtual machine scale set network profile.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
 * The list of network configurations.
 *
 * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile] The
 * virtual machine scale set diagnostics profile.
 *
 * @param {object}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
 * Diagnostics is a debugging feature which allows you to view Console Output
 * and Screenshot to diagnose VM status. <br><br> You can easily view the
 * output of your console log. <br><br> Azure also enables you to see a
 * screenshot of the VM from the hypervisor.
 *
 * @param {boolean}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
 * Whether boot diagnostics should be enabled on the Virtual Machine.
 *
 * @param {string}
 * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
 * Uri of the storage account to use for placing the console output and
 * screenshot.
 *
 * @param {object} [parameters.virtualMachineProfile.extensionProfile] The
 * virtual machine scale set extension profile.
 *
 * @param {array}
 * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
 * machine scale set child extension resources.
 *
 * @param {string} [parameters.virtualMachineProfile.licenseType] The license
 * type, which is for bring your own license scenario.
 *
 * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
 * Machine Scale Set should be overprovisioned.
 *
 * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
 * scale set to a single placement group, of max size 100 virtual machines.
 *
 * @param {object} [parameters.identity] The identity of the virtual machine
 * scale set, if configured.
 *
 * @param {string} [parameters.identity.type] The type of identity used for the
 * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
 * both an implicitly created identity and a set of user assigned identities.
 * The type 'None' will remove any identities from the virtual machine scale
 * set. Possible values include: 'SystemAssigned', 'UserAssigned',
 * 'SystemAssigned, UserAssigned', 'None'
 *
 * @param {object} [parameters.identity.userAssignedIdentities] The list of
 * user identities associated with the virtual machine scale set. The user
 * identity dictionary key references will be ARM resource ids in the form:
 * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
 *
 * @param {object} [parameters.tags] Resource tags
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSet} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginUpdate(resourceGroupName, vmScaleSetName, parameters, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (parameters === null || parameters === undefined) {
      throw new Error('parameters cannot be null or undefined.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'PATCH';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (parameters !== null && parameters !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetUpdate']().mapper();
      requestModel = client.serialize(requestModelMapper, parameters, 'parameters');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(parameters, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSet']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Deletes a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginDeleteMethod(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'DELETE';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202 && statusCode !== 204) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Deallocates specific virtual machines in a VM scale set. Shuts down the
 * virtual machines and releases the compute resources. You are not billed for
 * the compute resources that this virtual machine scale set deallocates.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginDeallocate(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let instanceIds = (options && options.instanceIds !== undefined) ? options.instanceIds : undefined;
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (Array.isArray(instanceIds)) {
      for (let i = 0; i < instanceIds.length; i++) {
        if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
          throw new Error('instanceIds[i] must be of type string.');
        }
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/deallocate';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Deletes virtual machines in a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {array} instanceIds The virtual machine scale set instance ids.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginDeleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (!Array.isArray(instanceIds)) {
      throw new Error('instanceIds cannot be null or undefined and it must be of type array.');
    }
    for (let i = 0; i < instanceIds.length; i++) {
      if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
        throw new Error('instanceIds[i] must be of type string.');
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceRequiredIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/delete';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceRequiredIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Power off (stop) one or more virtual machines in a VM scale set. Note that
 * resources are still attached and you are getting charged for the resources.
 * Instead, use deallocate to release resources and avoid charges.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginPowerOff(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let instanceIds = (options && options.instanceIds !== undefined) ? options.instanceIds : undefined;
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (Array.isArray(instanceIds)) {
      for (let i = 0; i < instanceIds.length; i++) {
        if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
          throw new Error('instanceIds[i] must be of type string.');
        }
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/poweroff';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Restarts one or more virtual machines in a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginRestart(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let instanceIds = (options && options.instanceIds !== undefined) ? options.instanceIds : undefined;
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (Array.isArray(instanceIds)) {
      for (let i = 0; i < instanceIds.length; i++) {
        if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
          throw new Error('instanceIds[i] must be of type string.');
        }
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/restart';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Starts one or more virtual machines in a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginStart(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let instanceIds = (options && options.instanceIds !== undefined) ? options.instanceIds : undefined;
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (Array.isArray(instanceIds)) {
      for (let i = 0; i < instanceIds.length; i++) {
        if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
          throw new Error('instanceIds[i] must be of type string.');
        }
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/start';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Redeploy one or more virtual machines in a VM scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginRedeploy(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let instanceIds = (options && options.instanceIds !== undefined) ? options.instanceIds : undefined;
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (Array.isArray(instanceIds)) {
      for (let i = 0; i < instanceIds.length; i++) {
        if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
          throw new Error('instanceIds[i] must be of type string.');
        }
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/redeploy';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Perform maintenance on one or more virtual machines in a VM scale set.
 * Operation on instances which are not eligible for perform maintenance will
 * be failed. Please refer to best practices for more details:
 * https://docs.microsoft.com/en-us/azure/virtual-machine-scale-sets/virtual-machine-scale-sets-maintenance-notifications
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginPerformMaintenance(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let instanceIds = (options && options.instanceIds !== undefined) ? options.instanceIds : undefined;
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (Array.isArray(instanceIds)) {
      for (let i = 0; i < instanceIds.length; i++) {
        if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
          throw new Error('instanceIds[i] must be of type string.');
        }
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/performMaintenance';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Upgrades one or more virtual machines to the latest SKU set in the VM scale
 * set model.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {array} instanceIds The virtual machine scale set instance ids.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginUpdateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (!Array.isArray(instanceIds)) {
      throw new Error('instanceIds cannot be null or undefined and it must be of type array.');
    }
    for (let i = 0; i < instanceIds.length; i++) {
      if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
        throw new Error('instanceIds[i] must be of type string.');
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceRequiredIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/manualupgrade';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceRequiredIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Reimages (upgrade the operating system) one or more virtual machines in a VM
 * scale set.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginReimage(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let instanceIds = (options && options.instanceIds !== undefined) ? options.instanceIds : undefined;
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (Array.isArray(instanceIds)) {
      for (let i = 0; i < instanceIds.length; i++) {
        if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
          throw new Error('instanceIds[i] must be of type string.');
        }
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/reimage';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Reimages all the disks ( including data disks ) in the virtual machines in a
 * VM scale set. This operation is only supported for managed disks.
 *
 * @param {string} resourceGroupName The name of the resource group.
 *
 * @param {string} vmScaleSetName The name of the VM scale set.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {array} [options.instanceIds] The virtual machine scale set instance
 * ids. Omitting the virtual machine scale set instance ids will result in the
 * operation being performed on all virtual machines in the virtual machine
 * scale set.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {null} [result]   - The deserialized result object if an error did not occur.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _beginReimageAll(resourceGroupName, vmScaleSetName, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  let instanceIds = (options && options.instanceIds !== undefined) ? options.instanceIds : undefined;
  let apiVersion = '2018-10-01';
  // Validate
  try {
    if (resourceGroupName === null || resourceGroupName === undefined || typeof resourceGroupName.valueOf() !== 'string') {
      throw new Error('resourceGroupName cannot be null or undefined and it must be of type string.');
    }
    if (vmScaleSetName === null || vmScaleSetName === undefined || typeof vmScaleSetName.valueOf() !== 'string') {
      throw new Error('vmScaleSetName cannot be null or undefined and it must be of type string.');
    }
    if (this.client.subscriptionId === null || this.client.subscriptionId === undefined || typeof this.client.subscriptionId.valueOf() !== 'string') {
      throw new Error('this.client.subscriptionId cannot be null or undefined and it must be of type string.');
    }
    if (Array.isArray(instanceIds)) {
      for (let i = 0; i < instanceIds.length; i++) {
        if (instanceIds[i] !== null && instanceIds[i] !== undefined && typeof instanceIds[i].valueOf() !== 'string') {
          throw new Error('instanceIds[i] must be of type string.');
        }
      }
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }
  let vmInstanceIDs;
  if (instanceIds !== null && instanceIds !== undefined) {
    vmInstanceIDs = new client.models['VirtualMachineScaleSetVMInstanceIDs']();
    vmInstanceIDs.instanceIds = instanceIds;
  }

  // Construct URL
  let baseUrl = this.client.baseUri;
  let requestUrl = baseUrl + (baseUrl.endsWith('/') ? '' : '/') + 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/virtualMachineScaleSets/{vmScaleSetName}/reimageall';
  requestUrl = requestUrl.replace('{resourceGroupName}', encodeURIComponent(resourceGroupName));
  requestUrl = requestUrl.replace('{vmScaleSetName}', encodeURIComponent(vmScaleSetName));
  requestUrl = requestUrl.replace('{subscriptionId}', encodeURIComponent(this.client.subscriptionId));
  let queryParameters = [];
  queryParameters.push('api-version=' + encodeURIComponent(apiVersion));
  if (queryParameters.length > 0) {
    requestUrl += '?' + queryParameters.join('&');
  }

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'POST';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  // Serialize Request
  let requestContent = null;
  let requestModel = null;
  try {
    if (vmInstanceIDs !== null && vmInstanceIDs !== undefined) {
      let requestModelMapper = new client.models['VirtualMachineScaleSetVMInstanceIDs']().mapper();
      requestModel = client.serialize(requestModelMapper, vmInstanceIDs, 'vmInstanceIDs');
      requestContent = JSON.stringify(requestModel);
    }
  } catch (error) {
    let serializationError = new Error(`Error "${error.message}" occurred in serializing the ` +
        `payload - ${JSON.stringify(vmInstanceIDs, null, 2)}.`);
    return callback(serializationError);
  }
  httpRequest.body = requestContent;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200 && statusCode !== 202) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets a list of all VM scale sets under a resource group.
 *
 * @param {string} nextPageLink The NextLink from the previous successful call
 * to List operation.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetListResult} for more
 *                      information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _listNext(nextPageLink, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  // Validate
  try {
    if (nextPageLink === null || nextPageLink === undefined || typeof nextPageLink.valueOf() !== 'string') {
      throw new Error('nextPageLink cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let requestUrl = '{nextLink}';
  requestUrl = requestUrl.replace('{nextLink}', nextPageLink);

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetListResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets a list of all VM Scale Sets in the subscription, regardless of the
 * associated resource group. Use nextLink property in the response to get the
 * next page of VM Scale Sets. Do this till nextLink is null to fetch all the
 * VM Scale Sets.
 *
 * @param {string} nextPageLink The NextLink from the previous successful call
 * to List operation.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetListWithLinkResult}
 *                      for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _listAllNext(nextPageLink, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  // Validate
  try {
    if (nextPageLink === null || nextPageLink === undefined || typeof nextPageLink.valueOf() !== 'string') {
      throw new Error('nextPageLink cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let requestUrl = '{nextLink}';
  requestUrl = requestUrl.replace('{nextLink}', nextPageLink);

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetListWithLinkResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets a list of SKUs available for your VM scale set, including the minimum
 * and maximum VM instances allowed for each SKU.
 *
 * @param {string} nextPageLink The NextLink from the previous successful call
 * to List operation.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetListSkusResult} for
 *                      more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _listSkusNext(nextPageLink, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  // Validate
  try {
    if (nextPageLink === null || nextPageLink === undefined || typeof nextPageLink.valueOf() !== 'string') {
      throw new Error('nextPageLink cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let requestUrl = '{nextLink}';
  requestUrl = requestUrl.replace('{nextLink}', nextPageLink);

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetListSkusResult']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/**
 * Gets list of OS upgrades on a VM scale set instance.
 *
 * @param {string} nextPageLink The NextLink from the previous successful call
 * to List operation.
 *
 * @param {object} [options] Optional Parameters.
 *
 * @param {object} [options.customHeaders] Headers that will be added to the
 * request
 *
 * @param {function} callback - The callback.
 *
 * @returns {function} callback(err, result, request, response)
 *
 *                      {Error}  err        - The Error object if an error occurred, null otherwise.
 *
 *                      {object} [result]   - The deserialized result object if an error did not occur.
 *                      See {@link VirtualMachineScaleSetListOSUpgradeHistory}
 *                      for more information.
 *
 *                      {object} [request]  - The HTTP Request object if an error did not occur.
 *
 *                      {stream} [response] - The HTTP Response stream if an error did not occur.
 */
function _getOSUpgradeHistoryNext(nextPageLink, options, callback) {
   /* jshint validthis: true */
  let client = this.client;
  if(!callback && typeof options === 'function') {
    callback = options;
    options = null;
  }
  if (!callback) {
    throw new Error('callback cannot be null.');
  }
  // Validate
  try {
    if (nextPageLink === null || nextPageLink === undefined || typeof nextPageLink.valueOf() !== 'string') {
      throw new Error('nextPageLink cannot be null or undefined and it must be of type string.');
    }
    if (this.client.acceptLanguage !== null && this.client.acceptLanguage !== undefined && typeof this.client.acceptLanguage.valueOf() !== 'string') {
      throw new Error('this.client.acceptLanguage must be of type string.');
    }
  } catch (error) {
    return callback(error);
  }

  // Construct URL
  let requestUrl = '{nextLink}';
  requestUrl = requestUrl.replace('{nextLink}', nextPageLink);

  // Create HTTP transport objects
  let httpRequest = new WebResource();
  httpRequest.method = 'GET';
  httpRequest.url = requestUrl;
  httpRequest.headers = {};
  // Set Headers
  httpRequest.headers['Content-Type'] = 'application/json; charset=utf-8';
  if (this.client.generateClientRequestId) {
      httpRequest.headers['x-ms-client-request-id'] = msRestAzure.generateUuid();
  }
  if (this.client.acceptLanguage !== undefined && this.client.acceptLanguage !== null) {
    httpRequest.headers['accept-language'] = this.client.acceptLanguage;
  }
  if(options) {
    for(let headerName in options['customHeaders']) {
      if (options['customHeaders'].hasOwnProperty(headerName)) {
        httpRequest.headers[headerName] = options['customHeaders'][headerName];
      }
    }
  }
  httpRequest.body = null;
  // Send Request
  return client.pipeline(httpRequest, (err, response, responseBody) => {
    if (err) {
      return callback(err);
    }
    let statusCode = response.statusCode;
    if (statusCode !== 200) {
      let error = new Error(responseBody);
      error.statusCode = response.statusCode;
      error.request = msRest.stripRequest(httpRequest);
      error.response = msRest.stripResponse(response);
      if (responseBody === '') responseBody = null;
      let parsedErrorResponse;
      try {
        parsedErrorResponse = JSON.parse(responseBody);
        if (parsedErrorResponse) {
          if (parsedErrorResponse.error) parsedErrorResponse = parsedErrorResponse.error;
          if (parsedErrorResponse.code) error.code = parsedErrorResponse.code;
          if (parsedErrorResponse.message) error.message = parsedErrorResponse.message;
        }
        if (parsedErrorResponse !== null && parsedErrorResponse !== undefined) {
          let resultMapper = new client.models['CloudError']().mapper();
          error.body = client.deserialize(resultMapper, parsedErrorResponse, 'error.body');
        }
      } catch (defaultError) {
        error.message = `Error "${defaultError.message}" occurred in deserializing the responseBody ` +
                         `- "${responseBody}" for the default response.`;
        return callback(error);
      }
      return callback(error);
    }
    // Create Result
    let result = null;
    if (responseBody === '') responseBody = null;
    // Deserialize Response
    if (statusCode === 200) {
      let parsedResponse = null;
      try {
        parsedResponse = JSON.parse(responseBody);
        result = JSON.parse(responseBody);
        if (parsedResponse !== null && parsedResponse !== undefined) {
          let resultMapper = new client.models['VirtualMachineScaleSetListOSUpgradeHistory']().mapper();
          result = client.deserialize(resultMapper, parsedResponse, 'result');
        }
      } catch (error) {
        let deserializationError = new Error(`Error ${error} occurred in deserializing the responseBody - ${responseBody}`);
        deserializationError.request = msRest.stripRequest(httpRequest);
        deserializationError.response = msRest.stripResponse(response);
        return callback(deserializationError);
      }
    }

    return callback(null, result, httpRequest, response);
  });
}

/** Class representing a VirtualMachineScaleSets. */
class VirtualMachineScaleSets {
  /**
   * Create a VirtualMachineScaleSets.
   * @param {ComputeManagementClient} client Reference to the service client.
   */
  constructor(client) {
    this.client = client;
    this._createOrUpdate = _createOrUpdate;
    this._update = _update;
    this._deleteMethod = _deleteMethod;
    this._get = _get;
    this._deallocate = _deallocate;
    this._deleteInstances = _deleteInstances;
    this._getInstanceView = _getInstanceView;
    this._list = _list;
    this._listAll = _listAll;
    this._listSkus = _listSkus;
    this._getOSUpgradeHistory = _getOSUpgradeHistory;
    this._powerOff = _powerOff;
    this._restart = _restart;
    this._start = _start;
    this._redeploy = _redeploy;
    this._performMaintenance = _performMaintenance;
    this._updateInstances = _updateInstances;
    this._reimage = _reimage;
    this._reimageAll = _reimageAll;
    this._forceRecoveryServiceFabricPlatformUpdateDomainWalk = _forceRecoveryServiceFabricPlatformUpdateDomainWalk;
    this._beginCreateOrUpdate = _beginCreateOrUpdate;
    this._beginUpdate = _beginUpdate;
    this._beginDeleteMethod = _beginDeleteMethod;
    this._beginDeallocate = _beginDeallocate;
    this._beginDeleteInstances = _beginDeleteInstances;
    this._beginPowerOff = _beginPowerOff;
    this._beginRestart = _beginRestart;
    this._beginStart = _beginStart;
    this._beginRedeploy = _beginRedeploy;
    this._beginPerformMaintenance = _beginPerformMaintenance;
    this._beginUpdateInstances = _beginUpdateInstances;
    this._beginReimage = _beginReimage;
    this._beginReimageAll = _beginReimageAll;
    this._listNext = _listNext;
    this._listAllNext = _listAllNext;
    this._listSkusNext = _listSkusNext;
    this._getOSUpgradeHistoryNext = _getOSUpgradeHistoryNext;
  }

  /**
   * Create or update a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set to create or
   * update.
   *
   * @param {object} parameters The scale set object.
   *
   * @param {object} [parameters.sku] The virtual machine scale set sku.
   *
   * @param {string} [parameters.sku.name] The sku name.
   *
   * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
   * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
   * /><br /> **Basic**
   *
   * @param {number} [parameters.sku.capacity] Specifies the number of virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.plan] Specifies information about the
   * marketplace image used to create the virtual machine. This element is only
   * used for marketplace images. Before you can use a marketplace image from an
   * API, you must enable the image for programmatic use.  In the Azure portal,
   * find the marketplace image that you want to use and then click **Want to
   * deploy programmatically, Get Started ->**. Enter any required information
   * and then click **Save**.
   *
   * @param {string} [parameters.plan.name] The plan ID.
   *
   * @param {string} [parameters.plan.publisher] The publisher ID.
   *
   * @param {string} [parameters.plan.product] Specifies the product of the image
   * from the marketplace. This is the same value as Offer under the
   * imageReference element.
   *
   * @param {string} [parameters.plan.promotionCode] The promotion code.
   *
   * @param {object} [parameters.upgradePolicy] The upgrade policy.
   *
   * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
   * upgrade to virtual machines in the scale set.<br /><br /> Possible values
   * are:<br /><br /> **Manual** - You  control the application of updates to
   * virtual machines in the scale set. You do this by using the manualUpgrade
   * action.<br /><br /> **Automatic** - All virtual machines in the scale set
   * are  automatically updated at the same time. Possible values include:
   * 'Automatic', 'Manual', 'Rolling'
   *
   * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
   * configuration parameters used while performing a rolling upgrade.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
   * maximum percent of total virtual machine instances that will be upgraded
   * simultaneously by the rolling upgrade in one batch. As this is a maximum,
   * unhealthy instances in previous or future batches can cause the percentage
   * of instances in a batch to decrease to ensure higher reliability. The
   * default value for this parameter is 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
   * The maximum percentage of the total virtual machine instances in the scale
   * set that can be simultaneously unhealthy, either as a result of being
   * upgraded, or by being found in an unhealthy state by the virtual machine
   * health checks before the rolling upgrade aborts. This constraint will be
   * checked prior to starting any batch. The default value for this parameter is
   * 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
   * The maximum percentage of upgraded virtual machine instances that can be
   * found to be in an unhealthy state. This check will happen after each batch
   * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
   * The default value for this parameter is 20%.
   *
   * @param {string}
   * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
   * wait time between completing the update for all virtual machines in one
   * batch and starting the next batch. The time duration should be specified in
   * ISO 8601 format. The default value is 0 seconds (PT0S).
   *
   * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
   * Configuration parameters used for performing automatic OS Upgrade.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
   * Whether OS upgrades should automatically be applied to scale set instances
   * in a rolling fashion when a newer version of the image becomes available.
   * Default value is false.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
   * Whether OS image rollback feature should be disabled. Default value is
   * false.
   *
   * @param {object} [parameters.virtualMachineProfile] The virtual machine
   * profile.
   *
   * @param {object} [parameters.virtualMachineProfile.osProfile] Specifies the
   * operating system settings for the virtual machines in the scale set.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.computerNamePrefix] Specifies
   * the computer name prefix for all of the virtual machines in the scale set.
   * Computer name prefixes must be 1 to 15 characters long.
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.adminUsername]
   * Specifies the name of the administrator account. <br><br> **Windows-only
   * restriction:** Cannot end in "." <br><br> **Disallowed values:**
   * "administrator", "admin", "user", "user1", "test", "user2", "test1",
   * "user3", "admin1", "1", "123", "a", "actuser", "adm", "admin2", "aspnet",
   * "backup", "console", "david", "guest", "john", "owner", "root", "server",
   * "sql", "support", "support_388945a0", "sys", "test2", "test3", "user4",
   * "user5". <br><br> **Minimum-length (Linux):** 1  character <br><br>
   * **Max-length (Linux):** 64 characters <br><br> **Max-length (Windows):** 20
   * characters  <br><br><li> For root access to the Linux VM, see [Using root
   * privileges on Linux virtual machines in
   * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-use-root-privileges?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)<br><li>
   * For a list of built-in system users on Linux that should not be used in this
   * field, see [Selecting User Names for Linux on
   * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-usernames?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.adminPassword]
   * Specifies the password of the administrator account. <br><br>
   * **Minimum-length (Windows):** 8 characters <br><br> **Minimum-length
   * (Linux):** 6 characters <br><br> **Max-length (Windows):** 123 characters
   * <br><br> **Max-length (Linux):** 72 characters <br><br> **Complexity
   * requirements:** 3 out of 4 conditions below need to be fulfilled <br> Has
   * lower characters <br>Has upper characters <br> Has a digit <br> Has a
   * special character (Regex match [\W_]) <br><br> **Disallowed values:**
   * "abc@123", "P@$$w0rd", "P@ssw0rd", "P@ssword123", "Pa$$word", "pass@word1",
   * "Password!", "Password1", "Password22", "iloveyou!" <br><br> For resetting
   * the password, see [How to reset the Remote Desktop service or its login
   * password in a Windows
   * VM](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-reset-rdp?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
   * <br><br> For resetting root password, see [Manage users, SSH, and check or
   * repair disks on Azure Linux VMs using the VMAccess
   * Extension](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-vmaccess-extension?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json#reset-root-password)
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.customData]
   * Specifies a base-64 encoded string of custom data. The base-64 encoded
   * string is decoded to a binary array that is saved as a file on the Virtual
   * Machine. The maximum length of the binary array is 65535 bytes. <br><br> For
   * using cloud-init for your VM, see [Using cloud-init to customize a Linux VM
   * during
   * creation](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-cloud-init?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] Specifies
   * Windows operating system settings on the virtual machine.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
   * Indicates whether virtual machine is enabled for automatic updates.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
   * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
   * Specifies additional base-64 encoded XML formatted information that can be
   * included in the Unattend.xml file, which is used by Windows Setup.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
   * Specifies the Windows Remote Management listeners. This enables remote
   * Windows PowerShell.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
   * The list of Windows Remote Management listeners
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] Specifies
   * the Linux operating system settings on the virtual machine. <br><br>For a
   * list of supported Linux distributions, see [Linux on Azure-Endorsed
   * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-endorsed-distros?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   * <br><br> For running non-endorsed distributions, see [Information for
   * Non-Endorsed
   * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-create-upload-generic?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json).
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
   * Specifies the ssh key configuration for a Linux OS.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
   * The list of SSH public keys used to authenticate with linux based VMs.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {array} [parameters.virtualMachineProfile.osProfile.secrets]
   * Specifies set of certificates that should be installed onto the virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile] Specifies
   * the storage settings for the virtual machine disks.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.imageReference] Specifies
   * information about the image to use. You can specify information about
   * platform images, marketplace images, or virtual machine images. This element
   * is required when you want to use a platform image, marketplace image, or
   * virtual machine image, but is not used in other creation operations.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
   * The image publisher.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
   * Specifies the offer of the platform image or marketplace image used to
   * create the virtual machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
   * image SKU.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
   * Specifies the version of the platform image or marketplace image used to
   * create the virtual machine. The allowed formats are Major.Minor.Build or
   * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
   * use the latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time even
   * if a new version becomes available.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
   * Id
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk]
   * Specifies information about the operating system disk used by the virtual
   * machines in the scale set. <br><br> For more information about disks, see
   * [About disks and VHDs for Azure virtual
   * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.name] The disk name.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] Specifies
   * the caching requirements. <br><br> Possible values are: <br><br> **None**
   * <br><br> **ReadOnly** <br><br> **ReadWrite** <br><br> Default: **None for
   * Standard storage. ReadOnly for Premium storage**. Possible values include:
   * 'None', 'ReadOnly', 'ReadWrite'
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
   * Specifies whether writeAccelerator should be enabled or disabled on the
   * disk.
   *
   * @param {string}
   * parameters.virtualMachineProfile.storageProfile.osDisk.createOption
   * Specifies how the virtual machines in the scale set should be
   * created.<br><br> The only allowed value is: **FromImage** \u2013 This value
   * is used when you are using an image to create the virtual machine. If you
   * are using a platform image, you also use the imageReference element
   * described above. If you are using a marketplace image, you  also use the
   * plan element previously described. Possible values include: 'FromImage',
   * 'Empty', 'Attach'
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings]
   * Specifies the differencing Disk Settings for the operating system disk used
   * by the virtual machine scale set.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings.option]
   * Specifies the differencing disk settings for operating system disk. Possible
   * values include: 'Local'
   *
   * @param {number}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
   * Specifies the size of the operating system disk in gigabytes. This element
   * can be used to overwrite the size of the disk in a virtual machine image.
   * <br><br> This value cannot be larger than 1023 GB
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.osType] This
   * property allows you to specify the type of the OS that is included in the
   * disk if creating a VM from user-image or a specialized VHD. <br><br>
   * Possible values are: <br><br> **Windows** <br><br> **Linux**. Possible
   * values include: 'Windows', 'Linux'
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image] Specifies
   * information about the unmanaged user image to base the scale set on.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
   * the virtual hard disk's uri.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers]
   * Specifies the container urls that are used to store operating system disks
   * for the scale set.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
   * managed disk parameters.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
   * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
   * can only be used with data disks, it cannot be used with OS Disk. Possible
   * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
   * 'UltraSSD_LRS'
   *
   * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
   * Specifies the parameters that are used to add data disks to the virtual
   * machines in the scale set. <br><br> For more information about disks, see
   * [About disks and VHDs for Azure virtual
   * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
   *
   * @param {object} [parameters.virtualMachineProfile.additionalCapabilities]
   * Specifies additional capabilities enabled or disabled on the virtual machine
   * in the scale set. For instance: whether the virtual machine has the
   * capability to support attaching managed data disks with UltraSSD_LRS storage
   * account type.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.additionalCapabilities.ultraSSDEnabled]
   * The flag that enables or disables a capability to have one or more managed
   * data disks with UltraSSD_LRS storage account type on the VM or VMSS. Managed
   * disks with storage account type UltraSSD_LRS can be added to a virtual
   * machine or virtual machine scale set only if this property is enabled.
   *
   * @param {object} [parameters.virtualMachineProfile.networkProfile] Specifies
   * properties of the network interfaces of the virtual machines in the scale
   * set.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.networkProfile.healthProbe] A reference to
   * a load balancer probe used to determine the health of an instance in the
   * virtual machine scale set. The reference will be in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/loadBalancers/{loadBalancerName}/probes/{probeName}'.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.networkProfile.healthProbe.id] The ARM
   * resource id in the form of
   * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
   *
   * @param {array}
   * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
   * The list of network configurations.
   *
   * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile]
   * Specifies the boot diagnostic settings state. <br><br>Minimum api-version:
   * 2015-06-15.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
   * Diagnostics is a debugging feature which allows you to view Console Output
   * and Screenshot to diagnose VM status. <br><br> You can easily view the
   * output of your console log. <br><br> Azure also enables you to see a
   * screenshot of the VM from the hypervisor.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
   * Whether boot diagnostics should be enabled on the Virtual Machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
   * Uri of the storage account to use for placing the console output and
   * screenshot.
   *
   * @param {object} [parameters.virtualMachineProfile.extensionProfile]
   * Specifies a collection of settings for extensions installed on virtual
   * machines in the scale set.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
   * machine scale set child extension resources.
   *
   * @param {string} [parameters.virtualMachineProfile.licenseType] Specifies
   * that the image or disk that is being used was licensed on-premises. This
   * element is only used for images that contain the Windows Server operating
   * system. <br><br> Possible values are: <br><br> Windows_Client <br><br>
   * Windows_Server <br><br> If this element is included in a request for an
   * update, the value must match the initial value. This value cannot be
   * updated. <br><br> For more information, see [Azure Hybrid Use Benefit for
   * Windows
   * Server](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-hybrid-use-benefit-licensing?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
   * <br><br> Minimum api-version: 2015-06-15
   *
   * @param {string} [parameters.virtualMachineProfile.priority] Specifies the
   * priority for the virtual machines in the scale set. <br><br>Minimum
   * api-version: 2017-10-30-preview. Possible values include: 'Regular', 'Low'
   *
   * @param {string} [parameters.virtualMachineProfile.evictionPolicy] Specifies
   * the eviction policy for virtual machines in a low priority scale set.
   * <br><br>Minimum api-version: 2017-10-30-preview. Possible values include:
   * 'Deallocate', 'Delete'
   *
   * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
   * Machine Scale Set should be overprovisioned.
   *
   * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
   * scale set to a single placement group, of max size 100 virtual machines.
   *
   * @param {boolean} [parameters.zoneBalance] Whether to force stictly even
   * Virtual Machine distribution cross x-zones in case there is zone outage.
   *
   * @param {number} [parameters.platformFaultDomainCount] Fault Domain count for
   * each placement group.
   *
   * @param {object} [parameters.identity] The identity of the virtual machine
   * scale set, if configured.
   *
   * @param {string} [parameters.identity.type] The type of identity used for the
   * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
   * both an implicitly created identity and a set of user assigned identities.
   * The type 'None' will remove any identities from the virtual machine scale
   * set. Possible values include: 'SystemAssigned', 'UserAssigned',
   * 'SystemAssigned, UserAssigned', 'None'
   *
   * @param {object} [parameters.identity.userAssignedIdentities] The list of
   * user identities associated with the virtual machine scale set. The user
   * identity dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {array} [parameters.zones] The virtual machine scale set zones.
   *
   * @param {string} parameters.location Resource location
   *
   * @param {object} [parameters.tags] Resource tags
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSet>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  createOrUpdateWithHttpOperationResponse(resourceGroupName, vmScaleSetName, parameters, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._createOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Create or update a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set to create or
   * update.
   *
   * @param {object} parameters The scale set object.
   *
   * @param {object} [parameters.sku] The virtual machine scale set sku.
   *
   * @param {string} [parameters.sku.name] The sku name.
   *
   * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
   * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
   * /><br /> **Basic**
   *
   * @param {number} [parameters.sku.capacity] Specifies the number of virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.plan] Specifies information about the
   * marketplace image used to create the virtual machine. This element is only
   * used for marketplace images. Before you can use a marketplace image from an
   * API, you must enable the image for programmatic use.  In the Azure portal,
   * find the marketplace image that you want to use and then click **Want to
   * deploy programmatically, Get Started ->**. Enter any required information
   * and then click **Save**.
   *
   * @param {string} [parameters.plan.name] The plan ID.
   *
   * @param {string} [parameters.plan.publisher] The publisher ID.
   *
   * @param {string} [parameters.plan.product] Specifies the product of the image
   * from the marketplace. This is the same value as Offer under the
   * imageReference element.
   *
   * @param {string} [parameters.plan.promotionCode] The promotion code.
   *
   * @param {object} [parameters.upgradePolicy] The upgrade policy.
   *
   * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
   * upgrade to virtual machines in the scale set.<br /><br /> Possible values
   * are:<br /><br /> **Manual** - You  control the application of updates to
   * virtual machines in the scale set. You do this by using the manualUpgrade
   * action.<br /><br /> **Automatic** - All virtual machines in the scale set
   * are  automatically updated at the same time. Possible values include:
   * 'Automatic', 'Manual', 'Rolling'
   *
   * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
   * configuration parameters used while performing a rolling upgrade.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
   * maximum percent of total virtual machine instances that will be upgraded
   * simultaneously by the rolling upgrade in one batch. As this is a maximum,
   * unhealthy instances in previous or future batches can cause the percentage
   * of instances in a batch to decrease to ensure higher reliability. The
   * default value for this parameter is 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
   * The maximum percentage of the total virtual machine instances in the scale
   * set that can be simultaneously unhealthy, either as a result of being
   * upgraded, or by being found in an unhealthy state by the virtual machine
   * health checks before the rolling upgrade aborts. This constraint will be
   * checked prior to starting any batch. The default value for this parameter is
   * 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
   * The maximum percentage of upgraded virtual machine instances that can be
   * found to be in an unhealthy state. This check will happen after each batch
   * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
   * The default value for this parameter is 20%.
   *
   * @param {string}
   * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
   * wait time between completing the update for all virtual machines in one
   * batch and starting the next batch. The time duration should be specified in
   * ISO 8601 format. The default value is 0 seconds (PT0S).
   *
   * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
   * Configuration parameters used for performing automatic OS Upgrade.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
   * Whether OS upgrades should automatically be applied to scale set instances
   * in a rolling fashion when a newer version of the image becomes available.
   * Default value is false.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
   * Whether OS image rollback feature should be disabled. Default value is
   * false.
   *
   * @param {object} [parameters.virtualMachineProfile] The virtual machine
   * profile.
   *
   * @param {object} [parameters.virtualMachineProfile.osProfile] Specifies the
   * operating system settings for the virtual machines in the scale set.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.computerNamePrefix] Specifies
   * the computer name prefix for all of the virtual machines in the scale set.
   * Computer name prefixes must be 1 to 15 characters long.
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.adminUsername]
   * Specifies the name of the administrator account. <br><br> **Windows-only
   * restriction:** Cannot end in "." <br><br> **Disallowed values:**
   * "administrator", "admin", "user", "user1", "test", "user2", "test1",
   * "user3", "admin1", "1", "123", "a", "actuser", "adm", "admin2", "aspnet",
   * "backup", "console", "david", "guest", "john", "owner", "root", "server",
   * "sql", "support", "support_388945a0", "sys", "test2", "test3", "user4",
   * "user5". <br><br> **Minimum-length (Linux):** 1  character <br><br>
   * **Max-length (Linux):** 64 characters <br><br> **Max-length (Windows):** 20
   * characters  <br><br><li> For root access to the Linux VM, see [Using root
   * privileges on Linux virtual machines in
   * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-use-root-privileges?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)<br><li>
   * For a list of built-in system users on Linux that should not be used in this
   * field, see [Selecting User Names for Linux on
   * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-usernames?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.adminPassword]
   * Specifies the password of the administrator account. <br><br>
   * **Minimum-length (Windows):** 8 characters <br><br> **Minimum-length
   * (Linux):** 6 characters <br><br> **Max-length (Windows):** 123 characters
   * <br><br> **Max-length (Linux):** 72 characters <br><br> **Complexity
   * requirements:** 3 out of 4 conditions below need to be fulfilled <br> Has
   * lower characters <br>Has upper characters <br> Has a digit <br> Has a
   * special character (Regex match [\W_]) <br><br> **Disallowed values:**
   * "abc@123", "P@$$w0rd", "P@ssw0rd", "P@ssword123", "Pa$$word", "pass@word1",
   * "Password!", "Password1", "Password22", "iloveyou!" <br><br> For resetting
   * the password, see [How to reset the Remote Desktop service or its login
   * password in a Windows
   * VM](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-reset-rdp?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
   * <br><br> For resetting root password, see [Manage users, SSH, and check or
   * repair disks on Azure Linux VMs using the VMAccess
   * Extension](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-vmaccess-extension?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json#reset-root-password)
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.customData]
   * Specifies a base-64 encoded string of custom data. The base-64 encoded
   * string is decoded to a binary array that is saved as a file on the Virtual
   * Machine. The maximum length of the binary array is 65535 bytes. <br><br> For
   * using cloud-init for your VM, see [Using cloud-init to customize a Linux VM
   * during
   * creation](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-cloud-init?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] Specifies
   * Windows operating system settings on the virtual machine.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
   * Indicates whether virtual machine is enabled for automatic updates.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
   * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
   * Specifies additional base-64 encoded XML formatted information that can be
   * included in the Unattend.xml file, which is used by Windows Setup.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
   * Specifies the Windows Remote Management listeners. This enables remote
   * Windows PowerShell.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
   * The list of Windows Remote Management listeners
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] Specifies
   * the Linux operating system settings on the virtual machine. <br><br>For a
   * list of supported Linux distributions, see [Linux on Azure-Endorsed
   * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-endorsed-distros?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   * <br><br> For running non-endorsed distributions, see [Information for
   * Non-Endorsed
   * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-create-upload-generic?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json).
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
   * Specifies the ssh key configuration for a Linux OS.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
   * The list of SSH public keys used to authenticate with linux based VMs.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {array} [parameters.virtualMachineProfile.osProfile.secrets]
   * Specifies set of certificates that should be installed onto the virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile] Specifies
   * the storage settings for the virtual machine disks.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.imageReference] Specifies
   * information about the image to use. You can specify information about
   * platform images, marketplace images, or virtual machine images. This element
   * is required when you want to use a platform image, marketplace image, or
   * virtual machine image, but is not used in other creation operations.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
   * The image publisher.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
   * Specifies the offer of the platform image or marketplace image used to
   * create the virtual machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
   * image SKU.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
   * Specifies the version of the platform image or marketplace image used to
   * create the virtual machine. The allowed formats are Major.Minor.Build or
   * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
   * use the latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time even
   * if a new version becomes available.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
   * Id
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk]
   * Specifies information about the operating system disk used by the virtual
   * machines in the scale set. <br><br> For more information about disks, see
   * [About disks and VHDs for Azure virtual
   * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.name] The disk name.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] Specifies
   * the caching requirements. <br><br> Possible values are: <br><br> **None**
   * <br><br> **ReadOnly** <br><br> **ReadWrite** <br><br> Default: **None for
   * Standard storage. ReadOnly for Premium storage**. Possible values include:
   * 'None', 'ReadOnly', 'ReadWrite'
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
   * Specifies whether writeAccelerator should be enabled or disabled on the
   * disk.
   *
   * @param {string}
   * parameters.virtualMachineProfile.storageProfile.osDisk.createOption
   * Specifies how the virtual machines in the scale set should be
   * created.<br><br> The only allowed value is: **FromImage** \u2013 This value
   * is used when you are using an image to create the virtual machine. If you
   * are using a platform image, you also use the imageReference element
   * described above. If you are using a marketplace image, you  also use the
   * plan element previously described. Possible values include: 'FromImage',
   * 'Empty', 'Attach'
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings]
   * Specifies the differencing Disk Settings for the operating system disk used
   * by the virtual machine scale set.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings.option]
   * Specifies the differencing disk settings for operating system disk. Possible
   * values include: 'Local'
   *
   * @param {number}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
   * Specifies the size of the operating system disk in gigabytes. This element
   * can be used to overwrite the size of the disk in a virtual machine image.
   * <br><br> This value cannot be larger than 1023 GB
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.osType] This
   * property allows you to specify the type of the OS that is included in the
   * disk if creating a VM from user-image or a specialized VHD. <br><br>
   * Possible values are: <br><br> **Windows** <br><br> **Linux**. Possible
   * values include: 'Windows', 'Linux'
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image] Specifies
   * information about the unmanaged user image to base the scale set on.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
   * the virtual hard disk's uri.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers]
   * Specifies the container urls that are used to store operating system disks
   * for the scale set.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
   * managed disk parameters.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
   * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
   * can only be used with data disks, it cannot be used with OS Disk. Possible
   * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
   * 'UltraSSD_LRS'
   *
   * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
   * Specifies the parameters that are used to add data disks to the virtual
   * machines in the scale set. <br><br> For more information about disks, see
   * [About disks and VHDs for Azure virtual
   * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
   *
   * @param {object} [parameters.virtualMachineProfile.additionalCapabilities]
   * Specifies additional capabilities enabled or disabled on the virtual machine
   * in the scale set. For instance: whether the virtual machine has the
   * capability to support attaching managed data disks with UltraSSD_LRS storage
   * account type.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.additionalCapabilities.ultraSSDEnabled]
   * The flag that enables or disables a capability to have one or more managed
   * data disks with UltraSSD_LRS storage account type on the VM or VMSS. Managed
   * disks with storage account type UltraSSD_LRS can be added to a virtual
   * machine or virtual machine scale set only if this property is enabled.
   *
   * @param {object} [parameters.virtualMachineProfile.networkProfile] Specifies
   * properties of the network interfaces of the virtual machines in the scale
   * set.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.networkProfile.healthProbe] A reference to
   * a load balancer probe used to determine the health of an instance in the
   * virtual machine scale set. The reference will be in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/loadBalancers/{loadBalancerName}/probes/{probeName}'.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.networkProfile.healthProbe.id] The ARM
   * resource id in the form of
   * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
   *
   * @param {array}
   * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
   * The list of network configurations.
   *
   * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile]
   * Specifies the boot diagnostic settings state. <br><br>Minimum api-version:
   * 2015-06-15.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
   * Diagnostics is a debugging feature which allows you to view Console Output
   * and Screenshot to diagnose VM status. <br><br> You can easily view the
   * output of your console log. <br><br> Azure also enables you to see a
   * screenshot of the VM from the hypervisor.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
   * Whether boot diagnostics should be enabled on the Virtual Machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
   * Uri of the storage account to use for placing the console output and
   * screenshot.
   *
   * @param {object} [parameters.virtualMachineProfile.extensionProfile]
   * Specifies a collection of settings for extensions installed on virtual
   * machines in the scale set.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
   * machine scale set child extension resources.
   *
   * @param {string} [parameters.virtualMachineProfile.licenseType] Specifies
   * that the image or disk that is being used was licensed on-premises. This
   * element is only used for images that contain the Windows Server operating
   * system. <br><br> Possible values are: <br><br> Windows_Client <br><br>
   * Windows_Server <br><br> If this element is included in a request for an
   * update, the value must match the initial value. This value cannot be
   * updated. <br><br> For more information, see [Azure Hybrid Use Benefit for
   * Windows
   * Server](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-hybrid-use-benefit-licensing?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
   * <br><br> Minimum api-version: 2015-06-15
   *
   * @param {string} [parameters.virtualMachineProfile.priority] Specifies the
   * priority for the virtual machines in the scale set. <br><br>Minimum
   * api-version: 2017-10-30-preview. Possible values include: 'Regular', 'Low'
   *
   * @param {string} [parameters.virtualMachineProfile.evictionPolicy] Specifies
   * the eviction policy for virtual machines in a low priority scale set.
   * <br><br>Minimum api-version: 2017-10-30-preview. Possible values include:
   * 'Deallocate', 'Delete'
   *
   * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
   * Machine Scale Set should be overprovisioned.
   *
   * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
   * scale set to a single placement group, of max size 100 virtual machines.
   *
   * @param {boolean} [parameters.zoneBalance] Whether to force stictly even
   * Virtual Machine distribution cross x-zones in case there is zone outage.
   *
   * @param {number} [parameters.platformFaultDomainCount] Fault Domain count for
   * each placement group.
   *
   * @param {object} [parameters.identity] The identity of the virtual machine
   * scale set, if configured.
   *
   * @param {string} [parameters.identity.type] The type of identity used for the
   * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
   * both an implicitly created identity and a set of user assigned identities.
   * The type 'None' will remove any identities from the virtual machine scale
   * set. Possible values include: 'SystemAssigned', 'UserAssigned',
   * 'SystemAssigned, UserAssigned', 'None'
   *
   * @param {object} [parameters.identity.userAssignedIdentities] The list of
   * user identities associated with the virtual machine scale set. The user
   * identity dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {array} [parameters.zones] The virtual machine scale set zones.
   *
   * @param {string} parameters.location Resource location
   *
   * @param {object} [parameters.tags] Resource tags
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSet} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSet} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  createOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._createOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._createOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, optionalCallback);
    }
  }

  /**
   * Update a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set to create or
   * update.
   *
   * @param {object} parameters The scale set object.
   *
   * @param {object} [parameters.sku] The virtual machine scale set sku.
   *
   * @param {string} [parameters.sku.name] The sku name.
   *
   * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
   * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
   * /><br /> **Basic**
   *
   * @param {number} [parameters.sku.capacity] Specifies the number of virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.plan] The purchase plan when deploying a virtual
   * machine scale set from VM Marketplace images.
   *
   * @param {string} [parameters.plan.name] The plan ID.
   *
   * @param {string} [parameters.plan.publisher] The publisher ID.
   *
   * @param {string} [parameters.plan.product] Specifies the product of the image
   * from the marketplace. This is the same value as Offer under the
   * imageReference element.
   *
   * @param {string} [parameters.plan.promotionCode] The promotion code.
   *
   * @param {object} [parameters.upgradePolicy] The upgrade policy.
   *
   * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
   * upgrade to virtual machines in the scale set.<br /><br /> Possible values
   * are:<br /><br /> **Manual** - You  control the application of updates to
   * virtual machines in the scale set. You do this by using the manualUpgrade
   * action.<br /><br /> **Automatic** - All virtual machines in the scale set
   * are  automatically updated at the same time. Possible values include:
   * 'Automatic', 'Manual', 'Rolling'
   *
   * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
   * configuration parameters used while performing a rolling upgrade.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
   * maximum percent of total virtual machine instances that will be upgraded
   * simultaneously by the rolling upgrade in one batch. As this is a maximum,
   * unhealthy instances in previous or future batches can cause the percentage
   * of instances in a batch to decrease to ensure higher reliability. The
   * default value for this parameter is 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
   * The maximum percentage of the total virtual machine instances in the scale
   * set that can be simultaneously unhealthy, either as a result of being
   * upgraded, or by being found in an unhealthy state by the virtual machine
   * health checks before the rolling upgrade aborts. This constraint will be
   * checked prior to starting any batch. The default value for this parameter is
   * 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
   * The maximum percentage of upgraded virtual machine instances that can be
   * found to be in an unhealthy state. This check will happen after each batch
   * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
   * The default value for this parameter is 20%.
   *
   * @param {string}
   * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
   * wait time between completing the update for all virtual machines in one
   * batch and starting the next batch. The time duration should be specified in
   * ISO 8601 format. The default value is 0 seconds (PT0S).
   *
   * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
   * Configuration parameters used for performing automatic OS Upgrade.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
   * Whether OS upgrades should automatically be applied to scale set instances
   * in a rolling fashion when a newer version of the image becomes available.
   * Default value is false.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
   * Whether OS image rollback feature should be disabled. Default value is
   * false.
   *
   * @param {object} [parameters.virtualMachineProfile] The virtual machine
   * profile.
   *
   * @param {object} [parameters.virtualMachineProfile.osProfile] The virtual
   * machine scale set OS profile.
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.customData] A
   * base-64 encoded string of custom data.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] The
   * Windows Configuration of the OS profile.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
   * Indicates whether virtual machine is enabled for automatic updates.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
   * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
   * Specifies additional base-64 encoded XML formatted information that can be
   * included in the Unattend.xml file, which is used by Windows Setup.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
   * Specifies the Windows Remote Management listeners. This enables remote
   * Windows PowerShell.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
   * The list of Windows Remote Management listeners
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] The Linux
   * Configuration of the OS profile.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
   * Specifies the ssh key configuration for a Linux OS.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
   * The list of SSH public keys used to authenticate with linux based VMs.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {array} [parameters.virtualMachineProfile.osProfile.secrets] The List
   * of certificates for addition to the VM.
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile] The
   * virtual machine scale set storage profile.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.imageReference] The image
   * reference.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
   * The image publisher.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
   * Specifies the offer of the platform image or marketplace image used to
   * create the virtual machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
   * image SKU.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
   * Specifies the version of the platform image or marketplace image used to
   * create the virtual machine. The allowed formats are Major.Minor.Build or
   * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
   * use the latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time even
   * if a new version becomes available.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
   * Id
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk] The
   * OS disk.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] The caching
   * type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
   * Specifies whether writeAccelerator should be enabled or disabled on the
   * disk.
   *
   * @param {number}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
   * Specifies the size of the operating system disk in gigabytes. This element
   * can be used to overwrite the size of the disk in a virtual machine image.
   * <br><br> This value cannot be larger than 1023 GB
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image] The Source
   * User Image VirtualHardDisk. This VirtualHardDisk will be copied before using
   * it to attach to the Virtual Machine. If SourceImage is provided, the
   * destination VirtualHardDisk should not exist.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
   * the virtual hard disk's uri.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers] The
   * list of virtual hard disk container uris.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
   * managed disk parameters.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
   * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
   * can only be used with data disks, it cannot be used with OS Disk. Possible
   * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
   * 'UltraSSD_LRS'
   *
   * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
   * The data disks.
   *
   * @param {object} [parameters.virtualMachineProfile.networkProfile] The
   * virtual machine scale set network profile.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
   * The list of network configurations.
   *
   * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile] The
   * virtual machine scale set diagnostics profile.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
   * Diagnostics is a debugging feature which allows you to view Console Output
   * and Screenshot to diagnose VM status. <br><br> You can easily view the
   * output of your console log. <br><br> Azure also enables you to see a
   * screenshot of the VM from the hypervisor.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
   * Whether boot diagnostics should be enabled on the Virtual Machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
   * Uri of the storage account to use for placing the console output and
   * screenshot.
   *
   * @param {object} [parameters.virtualMachineProfile.extensionProfile] The
   * virtual machine scale set extension profile.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
   * machine scale set child extension resources.
   *
   * @param {string} [parameters.virtualMachineProfile.licenseType] The license
   * type, which is for bring your own license scenario.
   *
   * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
   * Machine Scale Set should be overprovisioned.
   *
   * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
   * scale set to a single placement group, of max size 100 virtual machines.
   *
   * @param {object} [parameters.identity] The identity of the virtual machine
   * scale set, if configured.
   *
   * @param {string} [parameters.identity.type] The type of identity used for the
   * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
   * both an implicitly created identity and a set of user assigned identities.
   * The type 'None' will remove any identities from the virtual machine scale
   * set. Possible values include: 'SystemAssigned', 'UserAssigned',
   * 'SystemAssigned, UserAssigned', 'None'
   *
   * @param {object} [parameters.identity.userAssignedIdentities] The list of
   * user identities associated with the virtual machine scale set. The user
   * identity dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {object} [parameters.tags] Resource tags
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSet>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  updateWithHttpOperationResponse(resourceGroupName, vmScaleSetName, parameters, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._update(resourceGroupName, vmScaleSetName, parameters, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Update a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set to create or
   * update.
   *
   * @param {object} parameters The scale set object.
   *
   * @param {object} [parameters.sku] The virtual machine scale set sku.
   *
   * @param {string} [parameters.sku.name] The sku name.
   *
   * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
   * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
   * /><br /> **Basic**
   *
   * @param {number} [parameters.sku.capacity] Specifies the number of virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.plan] The purchase plan when deploying a virtual
   * machine scale set from VM Marketplace images.
   *
   * @param {string} [parameters.plan.name] The plan ID.
   *
   * @param {string} [parameters.plan.publisher] The publisher ID.
   *
   * @param {string} [parameters.plan.product] Specifies the product of the image
   * from the marketplace. This is the same value as Offer under the
   * imageReference element.
   *
   * @param {string} [parameters.plan.promotionCode] The promotion code.
   *
   * @param {object} [parameters.upgradePolicy] The upgrade policy.
   *
   * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
   * upgrade to virtual machines in the scale set.<br /><br /> Possible values
   * are:<br /><br /> **Manual** - You  control the application of updates to
   * virtual machines in the scale set. You do this by using the manualUpgrade
   * action.<br /><br /> **Automatic** - All virtual machines in the scale set
   * are  automatically updated at the same time. Possible values include:
   * 'Automatic', 'Manual', 'Rolling'
   *
   * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
   * configuration parameters used while performing a rolling upgrade.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
   * maximum percent of total virtual machine instances that will be upgraded
   * simultaneously by the rolling upgrade in one batch. As this is a maximum,
   * unhealthy instances in previous or future batches can cause the percentage
   * of instances in a batch to decrease to ensure higher reliability. The
   * default value for this parameter is 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
   * The maximum percentage of the total virtual machine instances in the scale
   * set that can be simultaneously unhealthy, either as a result of being
   * upgraded, or by being found in an unhealthy state by the virtual machine
   * health checks before the rolling upgrade aborts. This constraint will be
   * checked prior to starting any batch. The default value for this parameter is
   * 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
   * The maximum percentage of upgraded virtual machine instances that can be
   * found to be in an unhealthy state. This check will happen after each batch
   * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
   * The default value for this parameter is 20%.
   *
   * @param {string}
   * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
   * wait time between completing the update for all virtual machines in one
   * batch and starting the next batch. The time duration should be specified in
   * ISO 8601 format. The default value is 0 seconds (PT0S).
   *
   * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
   * Configuration parameters used for performing automatic OS Upgrade.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
   * Whether OS upgrades should automatically be applied to scale set instances
   * in a rolling fashion when a newer version of the image becomes available.
   * Default value is false.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
   * Whether OS image rollback feature should be disabled. Default value is
   * false.
   *
   * @param {object} [parameters.virtualMachineProfile] The virtual machine
   * profile.
   *
   * @param {object} [parameters.virtualMachineProfile.osProfile] The virtual
   * machine scale set OS profile.
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.customData] A
   * base-64 encoded string of custom data.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] The
   * Windows Configuration of the OS profile.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
   * Indicates whether virtual machine is enabled for automatic updates.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
   * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
   * Specifies additional base-64 encoded XML formatted information that can be
   * included in the Unattend.xml file, which is used by Windows Setup.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
   * Specifies the Windows Remote Management listeners. This enables remote
   * Windows PowerShell.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
   * The list of Windows Remote Management listeners
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] The Linux
   * Configuration of the OS profile.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
   * Specifies the ssh key configuration for a Linux OS.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
   * The list of SSH public keys used to authenticate with linux based VMs.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {array} [parameters.virtualMachineProfile.osProfile.secrets] The List
   * of certificates for addition to the VM.
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile] The
   * virtual machine scale set storage profile.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.imageReference] The image
   * reference.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
   * The image publisher.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
   * Specifies the offer of the platform image or marketplace image used to
   * create the virtual machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
   * image SKU.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
   * Specifies the version of the platform image or marketplace image used to
   * create the virtual machine. The allowed formats are Major.Minor.Build or
   * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
   * use the latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time even
   * if a new version becomes available.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
   * Id
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk] The
   * OS disk.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] The caching
   * type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
   * Specifies whether writeAccelerator should be enabled or disabled on the
   * disk.
   *
   * @param {number}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
   * Specifies the size of the operating system disk in gigabytes. This element
   * can be used to overwrite the size of the disk in a virtual machine image.
   * <br><br> This value cannot be larger than 1023 GB
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image] The Source
   * User Image VirtualHardDisk. This VirtualHardDisk will be copied before using
   * it to attach to the Virtual Machine. If SourceImage is provided, the
   * destination VirtualHardDisk should not exist.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
   * the virtual hard disk's uri.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers] The
   * list of virtual hard disk container uris.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
   * managed disk parameters.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
   * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
   * can only be used with data disks, it cannot be used with OS Disk. Possible
   * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
   * 'UltraSSD_LRS'
   *
   * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
   * The data disks.
   *
   * @param {object} [parameters.virtualMachineProfile.networkProfile] The
   * virtual machine scale set network profile.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
   * The list of network configurations.
   *
   * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile] The
   * virtual machine scale set diagnostics profile.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
   * Diagnostics is a debugging feature which allows you to view Console Output
   * and Screenshot to diagnose VM status. <br><br> You can easily view the
   * output of your console log. <br><br> Azure also enables you to see a
   * screenshot of the VM from the hypervisor.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
   * Whether boot diagnostics should be enabled on the Virtual Machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
   * Uri of the storage account to use for placing the console output and
   * screenshot.
   *
   * @param {object} [parameters.virtualMachineProfile.extensionProfile] The
   * virtual machine scale set extension profile.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
   * machine scale set child extension resources.
   *
   * @param {string} [parameters.virtualMachineProfile.licenseType] The license
   * type, which is for bring your own license scenario.
   *
   * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
   * Machine Scale Set should be overprovisioned.
   *
   * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
   * scale set to a single placement group, of max size 100 virtual machines.
   *
   * @param {object} [parameters.identity] The identity of the virtual machine
   * scale set, if configured.
   *
   * @param {string} [parameters.identity.type] The type of identity used for the
   * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
   * both an implicitly created identity and a set of user assigned identities.
   * The type 'None' will remove any identities from the virtual machine scale
   * set. Possible values include: 'SystemAssigned', 'UserAssigned',
   * 'SystemAssigned, UserAssigned', 'None'
   *
   * @param {object} [parameters.identity.userAssignedIdentities] The list of
   * user identities associated with the virtual machine scale set. The user
   * identity dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {object} [parameters.tags] Resource tags
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSet} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSet} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  update(resourceGroupName, vmScaleSetName, parameters, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._update(resourceGroupName, vmScaleSetName, parameters, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._update(resourceGroupName, vmScaleSetName, parameters, options, optionalCallback);
    }
  }

  /**
   * Deletes a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  deleteMethodWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._deleteMethod(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  deleteMethod(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._deleteMethod(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._deleteMethod(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Display information about a virtual machine scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSet>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._get(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Display information about a virtual machine scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSet} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSet} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  get(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._get(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._get(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Deallocates specific virtual machines in a VM scale set. Shuts down the
   * virtual machines and releases the compute resources. You are not billed for
   * the compute resources that this virtual machine scale set deallocates.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  deallocateWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._deallocate(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deallocates specific virtual machines in a VM scale set. Shuts down the
   * virtual machines and releases the compute resources. You are not billed for
   * the compute resources that this virtual machine scale set deallocates.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  deallocate(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._deallocate(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._deallocate(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Deletes virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {array} instanceIds The virtual machine scale set instance ids.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  deleteInstancesWithHttpOperationResponse(resourceGroupName, vmScaleSetName, instanceIds, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._deleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {array} instanceIds The virtual machine scale set instance ids.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  deleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._deleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._deleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, optionalCallback);
    }
  }

  /**
   * Gets the status of a VM scale set instance.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetInstanceView>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getInstanceViewWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._getInstanceView(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets the status of a VM scale set instance.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetInstanceView} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetInstanceView} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  getInstanceView(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._getInstanceView(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._getInstanceView(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Gets a list of all VM scale sets under a resource group.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetListResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listWithHttpOperationResponse(resourceGroupName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._list(resourceGroupName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets a list of all VM scale sets under a resource group.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetListResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetListResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  list(resourceGroupName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._list(resourceGroupName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._list(resourceGroupName, options, optionalCallback);
    }
  }

  /**
   * Gets a list of all VM Scale Sets in the subscription, regardless of the
   * associated resource group. Use nextLink property in the response to get the
   * next page of VM Scale Sets. Do this till nextLink is null to fetch all the
   * VM Scale Sets.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetListWithLinkResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listAllWithHttpOperationResponse(options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._listAll(options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets a list of all VM Scale Sets in the subscription, regardless of the
   * associated resource group. Use nextLink property in the response to get the
   * next page of VM Scale Sets. Do this till nextLink is null to fetch all the
   * VM Scale Sets.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetListWithLinkResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetListWithLinkResult}
   *                      for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  listAll(options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._listAll(options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._listAll(options, optionalCallback);
    }
  }

  /**
   * Gets a list of SKUs available for your VM scale set, including the minimum
   * and maximum VM instances allowed for each SKU.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetListSkusResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listSkusWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._listSkus(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets a list of SKUs available for your VM scale set, including the minimum
   * and maximum VM instances allowed for each SKU.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetListSkusResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetListSkusResult} for
   *                      more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  listSkus(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._listSkus(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._listSkus(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Gets list of OS upgrades on a VM scale set instance.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetListOSUpgradeHistory>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getOSUpgradeHistoryWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._getOSUpgradeHistory(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets list of OS upgrades on a VM scale set instance.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetListOSUpgradeHistory} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetListOSUpgradeHistory}
   *                      for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  getOSUpgradeHistory(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._getOSUpgradeHistory(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._getOSUpgradeHistory(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Power off (stop) one or more virtual machines in a VM scale set. Note that
   * resources are still attached and you are getting charged for the resources.
   * Instead, use deallocate to release resources and avoid charges.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  powerOffWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._powerOff(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Power off (stop) one or more virtual machines in a VM scale set. Note that
   * resources are still attached and you are getting charged for the resources.
   * Instead, use deallocate to release resources and avoid charges.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  powerOff(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._powerOff(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._powerOff(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Restarts one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  restartWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._restart(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Restarts one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  restart(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._restart(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._restart(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Starts one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  startWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._start(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Starts one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  start(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._start(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._start(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Redeploy one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  redeployWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._redeploy(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Redeploy one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  redeploy(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._redeploy(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._redeploy(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Perform maintenance on one or more virtual machines in a VM scale set.
   * Operation on instances which are not eligible for perform maintenance will
   * be failed. Please refer to best practices for more details:
   * https://docs.microsoft.com/en-us/azure/virtual-machine-scale-sets/virtual-machine-scale-sets-maintenance-notifications
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  performMaintenanceWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._performMaintenance(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Perform maintenance on one or more virtual machines in a VM scale set.
   * Operation on instances which are not eligible for perform maintenance will
   * be failed. Please refer to best practices for more details:
   * https://docs.microsoft.com/en-us/azure/virtual-machine-scale-sets/virtual-machine-scale-sets-maintenance-notifications
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  performMaintenance(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._performMaintenance(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._performMaintenance(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Upgrades one or more virtual machines to the latest SKU set in the VM scale
   * set model.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {array} instanceIds The virtual machine scale set instance ids.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  updateInstancesWithHttpOperationResponse(resourceGroupName, vmScaleSetName, instanceIds, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._updateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Upgrades one or more virtual machines to the latest SKU set in the VM scale
   * set model.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {array} instanceIds The virtual machine scale set instance ids.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  updateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._updateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._updateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, optionalCallback);
    }
  }

  /**
   * Reimages (upgrade the operating system) one or more virtual machines in a VM
   * scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  reimageWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._reimage(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Reimages (upgrade the operating system) one or more virtual machines in a VM
   * scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  reimage(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._reimage(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._reimage(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Reimages all the disks ( including data disks ) in the virtual machines in a
   * VM scale set. This operation is only supported for managed disks.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  reimageAllWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._reimageAll(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Reimages all the disks ( including data disks ) in the virtual machines in a
   * VM scale set. This operation is only supported for managed disks.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  reimageAll(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._reimageAll(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._reimageAll(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Manual platform update domain walk to update virtual machines in a service
   * fabric virtual machine scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {number} platformUpdateDomain The platform update domain for which a
   * manual recovery walk is requested
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<RecoveryWalkResponse>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  forceRecoveryServiceFabricPlatformUpdateDomainWalkWithHttpOperationResponse(resourceGroupName, vmScaleSetName, platformUpdateDomain, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._forceRecoveryServiceFabricPlatformUpdateDomainWalk(resourceGroupName, vmScaleSetName, platformUpdateDomain, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Manual platform update domain walk to update virtual machines in a service
   * fabric virtual machine scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {number} platformUpdateDomain The platform update domain for which a
   * manual recovery walk is requested
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {RecoveryWalkResponse} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link RecoveryWalkResponse} for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  forceRecoveryServiceFabricPlatformUpdateDomainWalk(resourceGroupName, vmScaleSetName, platformUpdateDomain, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._forceRecoveryServiceFabricPlatformUpdateDomainWalk(resourceGroupName, vmScaleSetName, platformUpdateDomain, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._forceRecoveryServiceFabricPlatformUpdateDomainWalk(resourceGroupName, vmScaleSetName, platformUpdateDomain, options, optionalCallback);
    }
  }

  /**
   * Create or update a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set to create or
   * update.
   *
   * @param {object} parameters The scale set object.
   *
   * @param {object} [parameters.sku] The virtual machine scale set sku.
   *
   * @param {string} [parameters.sku.name] The sku name.
   *
   * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
   * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
   * /><br /> **Basic**
   *
   * @param {number} [parameters.sku.capacity] Specifies the number of virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.plan] Specifies information about the
   * marketplace image used to create the virtual machine. This element is only
   * used for marketplace images. Before you can use a marketplace image from an
   * API, you must enable the image for programmatic use.  In the Azure portal,
   * find the marketplace image that you want to use and then click **Want to
   * deploy programmatically, Get Started ->**. Enter any required information
   * and then click **Save**.
   *
   * @param {string} [parameters.plan.name] The plan ID.
   *
   * @param {string} [parameters.plan.publisher] The publisher ID.
   *
   * @param {string} [parameters.plan.product] Specifies the product of the image
   * from the marketplace. This is the same value as Offer under the
   * imageReference element.
   *
   * @param {string} [parameters.plan.promotionCode] The promotion code.
   *
   * @param {object} [parameters.upgradePolicy] The upgrade policy.
   *
   * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
   * upgrade to virtual machines in the scale set.<br /><br /> Possible values
   * are:<br /><br /> **Manual** - You  control the application of updates to
   * virtual machines in the scale set. You do this by using the manualUpgrade
   * action.<br /><br /> **Automatic** - All virtual machines in the scale set
   * are  automatically updated at the same time. Possible values include:
   * 'Automatic', 'Manual', 'Rolling'
   *
   * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
   * configuration parameters used while performing a rolling upgrade.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
   * maximum percent of total virtual machine instances that will be upgraded
   * simultaneously by the rolling upgrade in one batch. As this is a maximum,
   * unhealthy instances in previous or future batches can cause the percentage
   * of instances in a batch to decrease to ensure higher reliability. The
   * default value for this parameter is 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
   * The maximum percentage of the total virtual machine instances in the scale
   * set that can be simultaneously unhealthy, either as a result of being
   * upgraded, or by being found in an unhealthy state by the virtual machine
   * health checks before the rolling upgrade aborts. This constraint will be
   * checked prior to starting any batch. The default value for this parameter is
   * 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
   * The maximum percentage of upgraded virtual machine instances that can be
   * found to be in an unhealthy state. This check will happen after each batch
   * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
   * The default value for this parameter is 20%.
   *
   * @param {string}
   * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
   * wait time between completing the update for all virtual machines in one
   * batch and starting the next batch. The time duration should be specified in
   * ISO 8601 format. The default value is 0 seconds (PT0S).
   *
   * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
   * Configuration parameters used for performing automatic OS Upgrade.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
   * Whether OS upgrades should automatically be applied to scale set instances
   * in a rolling fashion when a newer version of the image becomes available.
   * Default value is false.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
   * Whether OS image rollback feature should be disabled. Default value is
   * false.
   *
   * @param {object} [parameters.virtualMachineProfile] The virtual machine
   * profile.
   *
   * @param {object} [parameters.virtualMachineProfile.osProfile] Specifies the
   * operating system settings for the virtual machines in the scale set.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.computerNamePrefix] Specifies
   * the computer name prefix for all of the virtual machines in the scale set.
   * Computer name prefixes must be 1 to 15 characters long.
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.adminUsername]
   * Specifies the name of the administrator account. <br><br> **Windows-only
   * restriction:** Cannot end in "." <br><br> **Disallowed values:**
   * "administrator", "admin", "user", "user1", "test", "user2", "test1",
   * "user3", "admin1", "1", "123", "a", "actuser", "adm", "admin2", "aspnet",
   * "backup", "console", "david", "guest", "john", "owner", "root", "server",
   * "sql", "support", "support_388945a0", "sys", "test2", "test3", "user4",
   * "user5". <br><br> **Minimum-length (Linux):** 1  character <br><br>
   * **Max-length (Linux):** 64 characters <br><br> **Max-length (Windows):** 20
   * characters  <br><br><li> For root access to the Linux VM, see [Using root
   * privileges on Linux virtual machines in
   * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-use-root-privileges?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)<br><li>
   * For a list of built-in system users on Linux that should not be used in this
   * field, see [Selecting User Names for Linux on
   * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-usernames?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.adminPassword]
   * Specifies the password of the administrator account. <br><br>
   * **Minimum-length (Windows):** 8 characters <br><br> **Minimum-length
   * (Linux):** 6 characters <br><br> **Max-length (Windows):** 123 characters
   * <br><br> **Max-length (Linux):** 72 characters <br><br> **Complexity
   * requirements:** 3 out of 4 conditions below need to be fulfilled <br> Has
   * lower characters <br>Has upper characters <br> Has a digit <br> Has a
   * special character (Regex match [\W_]) <br><br> **Disallowed values:**
   * "abc@123", "P@$$w0rd", "P@ssw0rd", "P@ssword123", "Pa$$word", "pass@word1",
   * "Password!", "Password1", "Password22", "iloveyou!" <br><br> For resetting
   * the password, see [How to reset the Remote Desktop service or its login
   * password in a Windows
   * VM](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-reset-rdp?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
   * <br><br> For resetting root password, see [Manage users, SSH, and check or
   * repair disks on Azure Linux VMs using the VMAccess
   * Extension](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-vmaccess-extension?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json#reset-root-password)
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.customData]
   * Specifies a base-64 encoded string of custom data. The base-64 encoded
   * string is decoded to a binary array that is saved as a file on the Virtual
   * Machine. The maximum length of the binary array is 65535 bytes. <br><br> For
   * using cloud-init for your VM, see [Using cloud-init to customize a Linux VM
   * during
   * creation](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-cloud-init?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] Specifies
   * Windows operating system settings on the virtual machine.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
   * Indicates whether virtual machine is enabled for automatic updates.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
   * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
   * Specifies additional base-64 encoded XML formatted information that can be
   * included in the Unattend.xml file, which is used by Windows Setup.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
   * Specifies the Windows Remote Management listeners. This enables remote
   * Windows PowerShell.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
   * The list of Windows Remote Management listeners
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] Specifies
   * the Linux operating system settings on the virtual machine. <br><br>For a
   * list of supported Linux distributions, see [Linux on Azure-Endorsed
   * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-endorsed-distros?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   * <br><br> For running non-endorsed distributions, see [Information for
   * Non-Endorsed
   * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-create-upload-generic?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json).
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
   * Specifies the ssh key configuration for a Linux OS.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
   * The list of SSH public keys used to authenticate with linux based VMs.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {array} [parameters.virtualMachineProfile.osProfile.secrets]
   * Specifies set of certificates that should be installed onto the virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile] Specifies
   * the storage settings for the virtual machine disks.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.imageReference] Specifies
   * information about the image to use. You can specify information about
   * platform images, marketplace images, or virtual machine images. This element
   * is required when you want to use a platform image, marketplace image, or
   * virtual machine image, but is not used in other creation operations.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
   * The image publisher.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
   * Specifies the offer of the platform image or marketplace image used to
   * create the virtual machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
   * image SKU.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
   * Specifies the version of the platform image or marketplace image used to
   * create the virtual machine. The allowed formats are Major.Minor.Build or
   * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
   * use the latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time even
   * if a new version becomes available.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
   * Id
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk]
   * Specifies information about the operating system disk used by the virtual
   * machines in the scale set. <br><br> For more information about disks, see
   * [About disks and VHDs for Azure virtual
   * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.name] The disk name.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] Specifies
   * the caching requirements. <br><br> Possible values are: <br><br> **None**
   * <br><br> **ReadOnly** <br><br> **ReadWrite** <br><br> Default: **None for
   * Standard storage. ReadOnly for Premium storage**. Possible values include:
   * 'None', 'ReadOnly', 'ReadWrite'
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
   * Specifies whether writeAccelerator should be enabled or disabled on the
   * disk.
   *
   * @param {string}
   * parameters.virtualMachineProfile.storageProfile.osDisk.createOption
   * Specifies how the virtual machines in the scale set should be
   * created.<br><br> The only allowed value is: **FromImage** \u2013 This value
   * is used when you are using an image to create the virtual machine. If you
   * are using a platform image, you also use the imageReference element
   * described above. If you are using a marketplace image, you  also use the
   * plan element previously described. Possible values include: 'FromImage',
   * 'Empty', 'Attach'
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings]
   * Specifies the differencing Disk Settings for the operating system disk used
   * by the virtual machine scale set.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings.option]
   * Specifies the differencing disk settings for operating system disk. Possible
   * values include: 'Local'
   *
   * @param {number}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
   * Specifies the size of the operating system disk in gigabytes. This element
   * can be used to overwrite the size of the disk in a virtual machine image.
   * <br><br> This value cannot be larger than 1023 GB
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.osType] This
   * property allows you to specify the type of the OS that is included in the
   * disk if creating a VM from user-image or a specialized VHD. <br><br>
   * Possible values are: <br><br> **Windows** <br><br> **Linux**. Possible
   * values include: 'Windows', 'Linux'
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image] Specifies
   * information about the unmanaged user image to base the scale set on.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
   * the virtual hard disk's uri.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers]
   * Specifies the container urls that are used to store operating system disks
   * for the scale set.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
   * managed disk parameters.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
   * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
   * can only be used with data disks, it cannot be used with OS Disk. Possible
   * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
   * 'UltraSSD_LRS'
   *
   * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
   * Specifies the parameters that are used to add data disks to the virtual
   * machines in the scale set. <br><br> For more information about disks, see
   * [About disks and VHDs for Azure virtual
   * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
   *
   * @param {object} [parameters.virtualMachineProfile.additionalCapabilities]
   * Specifies additional capabilities enabled or disabled on the virtual machine
   * in the scale set. For instance: whether the virtual machine has the
   * capability to support attaching managed data disks with UltraSSD_LRS storage
   * account type.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.additionalCapabilities.ultraSSDEnabled]
   * The flag that enables or disables a capability to have one or more managed
   * data disks with UltraSSD_LRS storage account type on the VM or VMSS. Managed
   * disks with storage account type UltraSSD_LRS can be added to a virtual
   * machine or virtual machine scale set only if this property is enabled.
   *
   * @param {object} [parameters.virtualMachineProfile.networkProfile] Specifies
   * properties of the network interfaces of the virtual machines in the scale
   * set.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.networkProfile.healthProbe] A reference to
   * a load balancer probe used to determine the health of an instance in the
   * virtual machine scale set. The reference will be in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/loadBalancers/{loadBalancerName}/probes/{probeName}'.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.networkProfile.healthProbe.id] The ARM
   * resource id in the form of
   * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
   *
   * @param {array}
   * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
   * The list of network configurations.
   *
   * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile]
   * Specifies the boot diagnostic settings state. <br><br>Minimum api-version:
   * 2015-06-15.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
   * Diagnostics is a debugging feature which allows you to view Console Output
   * and Screenshot to diagnose VM status. <br><br> You can easily view the
   * output of your console log. <br><br> Azure also enables you to see a
   * screenshot of the VM from the hypervisor.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
   * Whether boot diagnostics should be enabled on the Virtual Machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
   * Uri of the storage account to use for placing the console output and
   * screenshot.
   *
   * @param {object} [parameters.virtualMachineProfile.extensionProfile]
   * Specifies a collection of settings for extensions installed on virtual
   * machines in the scale set.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
   * machine scale set child extension resources.
   *
   * @param {string} [parameters.virtualMachineProfile.licenseType] Specifies
   * that the image or disk that is being used was licensed on-premises. This
   * element is only used for images that contain the Windows Server operating
   * system. <br><br> Possible values are: <br><br> Windows_Client <br><br>
   * Windows_Server <br><br> If this element is included in a request for an
   * update, the value must match the initial value. This value cannot be
   * updated. <br><br> For more information, see [Azure Hybrid Use Benefit for
   * Windows
   * Server](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-hybrid-use-benefit-licensing?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
   * <br><br> Minimum api-version: 2015-06-15
   *
   * @param {string} [parameters.virtualMachineProfile.priority] Specifies the
   * priority for the virtual machines in the scale set. <br><br>Minimum
   * api-version: 2017-10-30-preview. Possible values include: 'Regular', 'Low'
   *
   * @param {string} [parameters.virtualMachineProfile.evictionPolicy] Specifies
   * the eviction policy for virtual machines in a low priority scale set.
   * <br><br>Minimum api-version: 2017-10-30-preview. Possible values include:
   * 'Deallocate', 'Delete'
   *
   * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
   * Machine Scale Set should be overprovisioned.
   *
   * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
   * scale set to a single placement group, of max size 100 virtual machines.
   *
   * @param {boolean} [parameters.zoneBalance] Whether to force stictly even
   * Virtual Machine distribution cross x-zones in case there is zone outage.
   *
   * @param {number} [parameters.platformFaultDomainCount] Fault Domain count for
   * each placement group.
   *
   * @param {object} [parameters.identity] The identity of the virtual machine
   * scale set, if configured.
   *
   * @param {string} [parameters.identity.type] The type of identity used for the
   * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
   * both an implicitly created identity and a set of user assigned identities.
   * The type 'None' will remove any identities from the virtual machine scale
   * set. Possible values include: 'SystemAssigned', 'UserAssigned',
   * 'SystemAssigned, UserAssigned', 'None'
   *
   * @param {object} [parameters.identity.userAssignedIdentities] The list of
   * user identities associated with the virtual machine scale set. The user
   * identity dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {array} [parameters.zones] The virtual machine scale set zones.
   *
   * @param {string} parameters.location Resource location
   *
   * @param {object} [parameters.tags] Resource tags
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSet>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginCreateOrUpdateWithHttpOperationResponse(resourceGroupName, vmScaleSetName, parameters, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginCreateOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Create or update a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set to create or
   * update.
   *
   * @param {object} parameters The scale set object.
   *
   * @param {object} [parameters.sku] The virtual machine scale set sku.
   *
   * @param {string} [parameters.sku.name] The sku name.
   *
   * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
   * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
   * /><br /> **Basic**
   *
   * @param {number} [parameters.sku.capacity] Specifies the number of virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.plan] Specifies information about the
   * marketplace image used to create the virtual machine. This element is only
   * used for marketplace images. Before you can use a marketplace image from an
   * API, you must enable the image for programmatic use.  In the Azure portal,
   * find the marketplace image that you want to use and then click **Want to
   * deploy programmatically, Get Started ->**. Enter any required information
   * and then click **Save**.
   *
   * @param {string} [parameters.plan.name] The plan ID.
   *
   * @param {string} [parameters.plan.publisher] The publisher ID.
   *
   * @param {string} [parameters.plan.product] Specifies the product of the image
   * from the marketplace. This is the same value as Offer under the
   * imageReference element.
   *
   * @param {string} [parameters.plan.promotionCode] The promotion code.
   *
   * @param {object} [parameters.upgradePolicy] The upgrade policy.
   *
   * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
   * upgrade to virtual machines in the scale set.<br /><br /> Possible values
   * are:<br /><br /> **Manual** - You  control the application of updates to
   * virtual machines in the scale set. You do this by using the manualUpgrade
   * action.<br /><br /> **Automatic** - All virtual machines in the scale set
   * are  automatically updated at the same time. Possible values include:
   * 'Automatic', 'Manual', 'Rolling'
   *
   * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
   * configuration parameters used while performing a rolling upgrade.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
   * maximum percent of total virtual machine instances that will be upgraded
   * simultaneously by the rolling upgrade in one batch. As this is a maximum,
   * unhealthy instances in previous or future batches can cause the percentage
   * of instances in a batch to decrease to ensure higher reliability. The
   * default value for this parameter is 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
   * The maximum percentage of the total virtual machine instances in the scale
   * set that can be simultaneously unhealthy, either as a result of being
   * upgraded, or by being found in an unhealthy state by the virtual machine
   * health checks before the rolling upgrade aborts. This constraint will be
   * checked prior to starting any batch. The default value for this parameter is
   * 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
   * The maximum percentage of upgraded virtual machine instances that can be
   * found to be in an unhealthy state. This check will happen after each batch
   * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
   * The default value for this parameter is 20%.
   *
   * @param {string}
   * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
   * wait time between completing the update for all virtual machines in one
   * batch and starting the next batch. The time duration should be specified in
   * ISO 8601 format. The default value is 0 seconds (PT0S).
   *
   * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
   * Configuration parameters used for performing automatic OS Upgrade.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
   * Whether OS upgrades should automatically be applied to scale set instances
   * in a rolling fashion when a newer version of the image becomes available.
   * Default value is false.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
   * Whether OS image rollback feature should be disabled. Default value is
   * false.
   *
   * @param {object} [parameters.virtualMachineProfile] The virtual machine
   * profile.
   *
   * @param {object} [parameters.virtualMachineProfile.osProfile] Specifies the
   * operating system settings for the virtual machines in the scale set.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.computerNamePrefix] Specifies
   * the computer name prefix for all of the virtual machines in the scale set.
   * Computer name prefixes must be 1 to 15 characters long.
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.adminUsername]
   * Specifies the name of the administrator account. <br><br> **Windows-only
   * restriction:** Cannot end in "." <br><br> **Disallowed values:**
   * "administrator", "admin", "user", "user1", "test", "user2", "test1",
   * "user3", "admin1", "1", "123", "a", "actuser", "adm", "admin2", "aspnet",
   * "backup", "console", "david", "guest", "john", "owner", "root", "server",
   * "sql", "support", "support_388945a0", "sys", "test2", "test3", "user4",
   * "user5". <br><br> **Minimum-length (Linux):** 1  character <br><br>
   * **Max-length (Linux):** 64 characters <br><br> **Max-length (Windows):** 20
   * characters  <br><br><li> For root access to the Linux VM, see [Using root
   * privileges on Linux virtual machines in
   * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-use-root-privileges?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)<br><li>
   * For a list of built-in system users on Linux that should not be used in this
   * field, see [Selecting User Names for Linux on
   * Azure](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-usernames?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.adminPassword]
   * Specifies the password of the administrator account. <br><br>
   * **Minimum-length (Windows):** 8 characters <br><br> **Minimum-length
   * (Linux):** 6 characters <br><br> **Max-length (Windows):** 123 characters
   * <br><br> **Max-length (Linux):** 72 characters <br><br> **Complexity
   * requirements:** 3 out of 4 conditions below need to be fulfilled <br> Has
   * lower characters <br>Has upper characters <br> Has a digit <br> Has a
   * special character (Regex match [\W_]) <br><br> **Disallowed values:**
   * "abc@123", "P@$$w0rd", "P@ssw0rd", "P@ssword123", "Pa$$word", "pass@word1",
   * "Password!", "Password1", "Password22", "iloveyou!" <br><br> For resetting
   * the password, see [How to reset the Remote Desktop service or its login
   * password in a Windows
   * VM](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-reset-rdp?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
   * <br><br> For resetting root password, see [Manage users, SSH, and check or
   * repair disks on Azure Linux VMs using the VMAccess
   * Extension](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-vmaccess-extension?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json#reset-root-password)
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.customData]
   * Specifies a base-64 encoded string of custom data. The base-64 encoded
   * string is decoded to a binary array that is saved as a file on the Virtual
   * Machine. The maximum length of the binary array is 65535 bytes. <br><br> For
   * using cloud-init for your VM, see [Using cloud-init to customize a Linux VM
   * during
   * creation](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-using-cloud-init?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] Specifies
   * Windows operating system settings on the virtual machine.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
   * Indicates whether virtual machine is enabled for automatic updates.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
   * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
   * Specifies additional base-64 encoded XML formatted information that can be
   * included in the Unattend.xml file, which is used by Windows Setup.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
   * Specifies the Windows Remote Management listeners. This enables remote
   * Windows PowerShell.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
   * The list of Windows Remote Management listeners
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] Specifies
   * the Linux operating system settings on the virtual machine. <br><br>For a
   * list of supported Linux distributions, see [Linux on Azure-Endorsed
   * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-endorsed-distros?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json)
   * <br><br> For running non-endorsed distributions, see [Information for
   * Non-Endorsed
   * Distributions](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-linux-create-upload-generic?toc=%2fazure%2fvirtual-machines%2flinux%2ftoc.json).
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
   * Specifies the ssh key configuration for a Linux OS.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
   * The list of SSH public keys used to authenticate with linux based VMs.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {array} [parameters.virtualMachineProfile.osProfile.secrets]
   * Specifies set of certificates that should be installed onto the virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile] Specifies
   * the storage settings for the virtual machine disks.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.imageReference] Specifies
   * information about the image to use. You can specify information about
   * platform images, marketplace images, or virtual machine images. This element
   * is required when you want to use a platform image, marketplace image, or
   * virtual machine image, but is not used in other creation operations.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
   * The image publisher.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
   * Specifies the offer of the platform image or marketplace image used to
   * create the virtual machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
   * image SKU.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
   * Specifies the version of the platform image or marketplace image used to
   * create the virtual machine. The allowed formats are Major.Minor.Build or
   * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
   * use the latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time even
   * if a new version becomes available.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
   * Id
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk]
   * Specifies information about the operating system disk used by the virtual
   * machines in the scale set. <br><br> For more information about disks, see
   * [About disks and VHDs for Azure virtual
   * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.name] The disk name.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] Specifies
   * the caching requirements. <br><br> Possible values are: <br><br> **None**
   * <br><br> **ReadOnly** <br><br> **ReadWrite** <br><br> Default: **None for
   * Standard storage. ReadOnly for Premium storage**. Possible values include:
   * 'None', 'ReadOnly', 'ReadWrite'
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
   * Specifies whether writeAccelerator should be enabled or disabled on the
   * disk.
   *
   * @param {string}
   * parameters.virtualMachineProfile.storageProfile.osDisk.createOption
   * Specifies how the virtual machines in the scale set should be
   * created.<br><br> The only allowed value is: **FromImage** \u2013 This value
   * is used when you are using an image to create the virtual machine. If you
   * are using a platform image, you also use the imageReference element
   * described above. If you are using a marketplace image, you  also use the
   * plan element previously described. Possible values include: 'FromImage',
   * 'Empty', 'Attach'
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings]
   * Specifies the differencing Disk Settings for the operating system disk used
   * by the virtual machine scale set.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diffDiskSettings.option]
   * Specifies the differencing disk settings for operating system disk. Possible
   * values include: 'Local'
   *
   * @param {number}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
   * Specifies the size of the operating system disk in gigabytes. This element
   * can be used to overwrite the size of the disk in a virtual machine image.
   * <br><br> This value cannot be larger than 1023 GB
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.osType] This
   * property allows you to specify the type of the OS that is included in the
   * disk if creating a VM from user-image or a specialized VHD. <br><br>
   * Possible values are: <br><br> **Windows** <br><br> **Linux**. Possible
   * values include: 'Windows', 'Linux'
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image] Specifies
   * information about the unmanaged user image to base the scale set on.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
   * the virtual hard disk's uri.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers]
   * Specifies the container urls that are used to store operating system disks
   * for the scale set.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
   * managed disk parameters.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
   * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
   * can only be used with data disks, it cannot be used with OS Disk. Possible
   * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
   * 'UltraSSD_LRS'
   *
   * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
   * Specifies the parameters that are used to add data disks to the virtual
   * machines in the scale set. <br><br> For more information about disks, see
   * [About disks and VHDs for Azure virtual
   * machines](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-about-disks-vhds?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json).
   *
   * @param {object} [parameters.virtualMachineProfile.additionalCapabilities]
   * Specifies additional capabilities enabled or disabled on the virtual machine
   * in the scale set. For instance: whether the virtual machine has the
   * capability to support attaching managed data disks with UltraSSD_LRS storage
   * account type.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.additionalCapabilities.ultraSSDEnabled]
   * The flag that enables or disables a capability to have one or more managed
   * data disks with UltraSSD_LRS storage account type on the VM or VMSS. Managed
   * disks with storage account type UltraSSD_LRS can be added to a virtual
   * machine or virtual machine scale set only if this property is enabled.
   *
   * @param {object} [parameters.virtualMachineProfile.networkProfile] Specifies
   * properties of the network interfaces of the virtual machines in the scale
   * set.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.networkProfile.healthProbe] A reference to
   * a load balancer probe used to determine the health of an instance in the
   * virtual machine scale set. The reference will be in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Network/loadBalancers/{loadBalancerName}/probes/{probeName}'.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.networkProfile.healthProbe.id] The ARM
   * resource id in the form of
   * /subscriptions/{SubcriptionId}/resourceGroups/{ResourceGroupName}/...
   *
   * @param {array}
   * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
   * The list of network configurations.
   *
   * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile]
   * Specifies the boot diagnostic settings state. <br><br>Minimum api-version:
   * 2015-06-15.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
   * Diagnostics is a debugging feature which allows you to view Console Output
   * and Screenshot to diagnose VM status. <br><br> You can easily view the
   * output of your console log. <br><br> Azure also enables you to see a
   * screenshot of the VM from the hypervisor.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
   * Whether boot diagnostics should be enabled on the Virtual Machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
   * Uri of the storage account to use for placing the console output and
   * screenshot.
   *
   * @param {object} [parameters.virtualMachineProfile.extensionProfile]
   * Specifies a collection of settings for extensions installed on virtual
   * machines in the scale set.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
   * machine scale set child extension resources.
   *
   * @param {string} [parameters.virtualMachineProfile.licenseType] Specifies
   * that the image or disk that is being used was licensed on-premises. This
   * element is only used for images that contain the Windows Server operating
   * system. <br><br> Possible values are: <br><br> Windows_Client <br><br>
   * Windows_Server <br><br> If this element is included in a request for an
   * update, the value must match the initial value. This value cannot be
   * updated. <br><br> For more information, see [Azure Hybrid Use Benefit for
   * Windows
   * Server](https://docs.microsoft.com/azure/virtual-machines/virtual-machines-windows-hybrid-use-benefit-licensing?toc=%2fazure%2fvirtual-machines%2fwindows%2ftoc.json)
   * <br><br> Minimum api-version: 2015-06-15
   *
   * @param {string} [parameters.virtualMachineProfile.priority] Specifies the
   * priority for the virtual machines in the scale set. <br><br>Minimum
   * api-version: 2017-10-30-preview. Possible values include: 'Regular', 'Low'
   *
   * @param {string} [parameters.virtualMachineProfile.evictionPolicy] Specifies
   * the eviction policy for virtual machines in a low priority scale set.
   * <br><br>Minimum api-version: 2017-10-30-preview. Possible values include:
   * 'Deallocate', 'Delete'
   *
   * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
   * Machine Scale Set should be overprovisioned.
   *
   * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
   * scale set to a single placement group, of max size 100 virtual machines.
   *
   * @param {boolean} [parameters.zoneBalance] Whether to force stictly even
   * Virtual Machine distribution cross x-zones in case there is zone outage.
   *
   * @param {number} [parameters.platformFaultDomainCount] Fault Domain count for
   * each placement group.
   *
   * @param {object} [parameters.identity] The identity of the virtual machine
   * scale set, if configured.
   *
   * @param {string} [parameters.identity.type] The type of identity used for the
   * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
   * both an implicitly created identity and a set of user assigned identities.
   * The type 'None' will remove any identities from the virtual machine scale
   * set. Possible values include: 'SystemAssigned', 'UserAssigned',
   * 'SystemAssigned, UserAssigned', 'None'
   *
   * @param {object} [parameters.identity.userAssignedIdentities] The list of
   * user identities associated with the virtual machine scale set. The user
   * identity dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {array} [parameters.zones] The virtual machine scale set zones.
   *
   * @param {string} parameters.location Resource location
   *
   * @param {object} [parameters.tags] Resource tags
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSet} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSet} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginCreateOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginCreateOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginCreateOrUpdate(resourceGroupName, vmScaleSetName, parameters, options, optionalCallback);
    }
  }

  /**
   * Update a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set to create or
   * update.
   *
   * @param {object} parameters The scale set object.
   *
   * @param {object} [parameters.sku] The virtual machine scale set sku.
   *
   * @param {string} [parameters.sku.name] The sku name.
   *
   * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
   * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
   * /><br /> **Basic**
   *
   * @param {number} [parameters.sku.capacity] Specifies the number of virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.plan] The purchase plan when deploying a virtual
   * machine scale set from VM Marketplace images.
   *
   * @param {string} [parameters.plan.name] The plan ID.
   *
   * @param {string} [parameters.plan.publisher] The publisher ID.
   *
   * @param {string} [parameters.plan.product] Specifies the product of the image
   * from the marketplace. This is the same value as Offer under the
   * imageReference element.
   *
   * @param {string} [parameters.plan.promotionCode] The promotion code.
   *
   * @param {object} [parameters.upgradePolicy] The upgrade policy.
   *
   * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
   * upgrade to virtual machines in the scale set.<br /><br /> Possible values
   * are:<br /><br /> **Manual** - You  control the application of updates to
   * virtual machines in the scale set. You do this by using the manualUpgrade
   * action.<br /><br /> **Automatic** - All virtual machines in the scale set
   * are  automatically updated at the same time. Possible values include:
   * 'Automatic', 'Manual', 'Rolling'
   *
   * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
   * configuration parameters used while performing a rolling upgrade.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
   * maximum percent of total virtual machine instances that will be upgraded
   * simultaneously by the rolling upgrade in one batch. As this is a maximum,
   * unhealthy instances in previous or future batches can cause the percentage
   * of instances in a batch to decrease to ensure higher reliability. The
   * default value for this parameter is 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
   * The maximum percentage of the total virtual machine instances in the scale
   * set that can be simultaneously unhealthy, either as a result of being
   * upgraded, or by being found in an unhealthy state by the virtual machine
   * health checks before the rolling upgrade aborts. This constraint will be
   * checked prior to starting any batch. The default value for this parameter is
   * 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
   * The maximum percentage of upgraded virtual machine instances that can be
   * found to be in an unhealthy state. This check will happen after each batch
   * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
   * The default value for this parameter is 20%.
   *
   * @param {string}
   * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
   * wait time between completing the update for all virtual machines in one
   * batch and starting the next batch. The time duration should be specified in
   * ISO 8601 format. The default value is 0 seconds (PT0S).
   *
   * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
   * Configuration parameters used for performing automatic OS Upgrade.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
   * Whether OS upgrades should automatically be applied to scale set instances
   * in a rolling fashion when a newer version of the image becomes available.
   * Default value is false.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
   * Whether OS image rollback feature should be disabled. Default value is
   * false.
   *
   * @param {object} [parameters.virtualMachineProfile] The virtual machine
   * profile.
   *
   * @param {object} [parameters.virtualMachineProfile.osProfile] The virtual
   * machine scale set OS profile.
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.customData] A
   * base-64 encoded string of custom data.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] The
   * Windows Configuration of the OS profile.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
   * Indicates whether virtual machine is enabled for automatic updates.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
   * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
   * Specifies additional base-64 encoded XML formatted information that can be
   * included in the Unattend.xml file, which is used by Windows Setup.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
   * Specifies the Windows Remote Management listeners. This enables remote
   * Windows PowerShell.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
   * The list of Windows Remote Management listeners
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] The Linux
   * Configuration of the OS profile.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
   * Specifies the ssh key configuration for a Linux OS.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
   * The list of SSH public keys used to authenticate with linux based VMs.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {array} [parameters.virtualMachineProfile.osProfile.secrets] The List
   * of certificates for addition to the VM.
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile] The
   * virtual machine scale set storage profile.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.imageReference] The image
   * reference.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
   * The image publisher.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
   * Specifies the offer of the platform image or marketplace image used to
   * create the virtual machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
   * image SKU.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
   * Specifies the version of the platform image or marketplace image used to
   * create the virtual machine. The allowed formats are Major.Minor.Build or
   * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
   * use the latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time even
   * if a new version becomes available.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
   * Id
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk] The
   * OS disk.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] The caching
   * type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
   * Specifies whether writeAccelerator should be enabled or disabled on the
   * disk.
   *
   * @param {number}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
   * Specifies the size of the operating system disk in gigabytes. This element
   * can be used to overwrite the size of the disk in a virtual machine image.
   * <br><br> This value cannot be larger than 1023 GB
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image] The Source
   * User Image VirtualHardDisk. This VirtualHardDisk will be copied before using
   * it to attach to the Virtual Machine. If SourceImage is provided, the
   * destination VirtualHardDisk should not exist.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
   * the virtual hard disk's uri.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers] The
   * list of virtual hard disk container uris.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
   * managed disk parameters.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
   * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
   * can only be used with data disks, it cannot be used with OS Disk. Possible
   * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
   * 'UltraSSD_LRS'
   *
   * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
   * The data disks.
   *
   * @param {object} [parameters.virtualMachineProfile.networkProfile] The
   * virtual machine scale set network profile.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
   * The list of network configurations.
   *
   * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile] The
   * virtual machine scale set diagnostics profile.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
   * Diagnostics is a debugging feature which allows you to view Console Output
   * and Screenshot to diagnose VM status. <br><br> You can easily view the
   * output of your console log. <br><br> Azure also enables you to see a
   * screenshot of the VM from the hypervisor.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
   * Whether boot diagnostics should be enabled on the Virtual Machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
   * Uri of the storage account to use for placing the console output and
   * screenshot.
   *
   * @param {object} [parameters.virtualMachineProfile.extensionProfile] The
   * virtual machine scale set extension profile.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
   * machine scale set child extension resources.
   *
   * @param {string} [parameters.virtualMachineProfile.licenseType] The license
   * type, which is for bring your own license scenario.
   *
   * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
   * Machine Scale Set should be overprovisioned.
   *
   * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
   * scale set to a single placement group, of max size 100 virtual machines.
   *
   * @param {object} [parameters.identity] The identity of the virtual machine
   * scale set, if configured.
   *
   * @param {string} [parameters.identity.type] The type of identity used for the
   * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
   * both an implicitly created identity and a set of user assigned identities.
   * The type 'None' will remove any identities from the virtual machine scale
   * set. Possible values include: 'SystemAssigned', 'UserAssigned',
   * 'SystemAssigned, UserAssigned', 'None'
   *
   * @param {object} [parameters.identity.userAssignedIdentities] The list of
   * user identities associated with the virtual machine scale set. The user
   * identity dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {object} [parameters.tags] Resource tags
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSet>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginUpdateWithHttpOperationResponse(resourceGroupName, vmScaleSetName, parameters, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginUpdate(resourceGroupName, vmScaleSetName, parameters, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Update a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set to create or
   * update.
   *
   * @param {object} parameters The scale set object.
   *
   * @param {object} [parameters.sku] The virtual machine scale set sku.
   *
   * @param {string} [parameters.sku.name] The sku name.
   *
   * @param {string} [parameters.sku.tier] Specifies the tier of virtual machines
   * in a scale set.<br /><br /> Possible Values:<br /><br /> **Standard**<br
   * /><br /> **Basic**
   *
   * @param {number} [parameters.sku.capacity] Specifies the number of virtual
   * machines in the scale set.
   *
   * @param {object} [parameters.plan] The purchase plan when deploying a virtual
   * machine scale set from VM Marketplace images.
   *
   * @param {string} [parameters.plan.name] The plan ID.
   *
   * @param {string} [parameters.plan.publisher] The publisher ID.
   *
   * @param {string} [parameters.plan.product] Specifies the product of the image
   * from the marketplace. This is the same value as Offer under the
   * imageReference element.
   *
   * @param {string} [parameters.plan.promotionCode] The promotion code.
   *
   * @param {object} [parameters.upgradePolicy] The upgrade policy.
   *
   * @param {string} [parameters.upgradePolicy.mode] Specifies the mode of an
   * upgrade to virtual machines in the scale set.<br /><br /> Possible values
   * are:<br /><br /> **Manual** - You  control the application of updates to
   * virtual machines in the scale set. You do this by using the manualUpgrade
   * action.<br /><br /> **Automatic** - All virtual machines in the scale set
   * are  automatically updated at the same time. Possible values include:
   * 'Automatic', 'Manual', 'Rolling'
   *
   * @param {object} [parameters.upgradePolicy.rollingUpgradePolicy] The
   * configuration parameters used while performing a rolling upgrade.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxBatchInstancePercent] The
   * maximum percent of total virtual machine instances that will be upgraded
   * simultaneously by the rolling upgrade in one batch. As this is a maximum,
   * unhealthy instances in previous or future batches can cause the percentage
   * of instances in a batch to decrease to ensure higher reliability. The
   * default value for this parameter is 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyInstancePercent]
   * The maximum percentage of the total virtual machine instances in the scale
   * set that can be simultaneously unhealthy, either as a result of being
   * upgraded, or by being found in an unhealthy state by the virtual machine
   * health checks before the rolling upgrade aborts. This constraint will be
   * checked prior to starting any batch. The default value for this parameter is
   * 20%.
   *
   * @param {number}
   * [parameters.upgradePolicy.rollingUpgradePolicy.maxUnhealthyUpgradedInstancePercent]
   * The maximum percentage of upgraded virtual machine instances that can be
   * found to be in an unhealthy state. This check will happen after each batch
   * is upgraded. If this percentage is ever exceeded, the rolling update aborts.
   * The default value for this parameter is 20%.
   *
   * @param {string}
   * [parameters.upgradePolicy.rollingUpgradePolicy.pauseTimeBetweenBatches] The
   * wait time between completing the update for all virtual machines in one
   * batch and starting the next batch. The time duration should be specified in
   * ISO 8601 format. The default value is 0 seconds (PT0S).
   *
   * @param {object} [parameters.upgradePolicy.automaticOSUpgradePolicy]
   * Configuration parameters used for performing automatic OS Upgrade.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.enableAutomaticOSUpgrade]
   * Whether OS upgrades should automatically be applied to scale set instances
   * in a rolling fashion when a newer version of the image becomes available.
   * Default value is false.
   *
   * @param {boolean}
   * [parameters.upgradePolicy.automaticOSUpgradePolicy.disableAutomaticRollback]
   * Whether OS image rollback feature should be disabled. Default value is
   * false.
   *
   * @param {object} [parameters.virtualMachineProfile] The virtual machine
   * profile.
   *
   * @param {object} [parameters.virtualMachineProfile.osProfile] The virtual
   * machine scale set OS profile.
   *
   * @param {string} [parameters.virtualMachineProfile.osProfile.customData] A
   * base-64 encoded string of custom data.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration] The
   * Windows Configuration of the OS profile.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.enableAutomaticUpdates]
   * Indicates whether virtual machine is enabled for automatic updates.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.timeZone]
   * Specifies the time zone of the virtual machine. e.g. "Pacific Standard Time"
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.additionalUnattendContent]
   * Specifies additional base-64 encoded XML formatted information that can be
   * included in the Unattend.xml file, which is used by Windows Setup.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM]
   * Specifies the Windows Remote Management listeners. This enables remote
   * Windows PowerShell.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.windowsConfiguration.winRM.listeners]
   * The list of Windows Remote Management listeners
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration] The Linux
   * Configuration of the OS profile.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.disablePasswordAuthentication]
   * Specifies whether password authentication should be disabled.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh]
   * Specifies the ssh key configuration for a Linux OS.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.ssh.publicKeys]
   * The list of SSH public keys used to authenticate with linux based VMs.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.osProfile.linuxConfiguration.provisionVMAgent]
   * Indicates whether virtual machine agent should be provisioned on the virtual
   * machine. <br><br> When this property is not specified in the request body,
   * default behavior is to set it to true.  This will ensure that VM Agent is
   * installed on the VM so that extensions can be added to the VM later.
   *
   * @param {array} [parameters.virtualMachineProfile.osProfile.secrets] The List
   * of certificates for addition to the VM.
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile] The
   * virtual machine scale set storage profile.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.imageReference] The image
   * reference.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.publisher]
   * The image publisher.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.offer]
   * Specifies the offer of the platform image or marketplace image used to
   * create the virtual machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.sku] The
   * image SKU.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.version]
   * Specifies the version of the platform image or marketplace image used to
   * create the virtual machine. The allowed formats are Major.Minor.Build or
   * 'latest'. Major, Minor, and Build are decimal numbers. Specify 'latest' to
   * use the latest version of an image available at deploy time. Even if you use
   * 'latest', the VM image will not automatically update after deploy time even
   * if a new version becomes available.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.imageReference.id] Resource
   * Id
   *
   * @param {object} [parameters.virtualMachineProfile.storageProfile.osDisk] The
   * OS disk.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.caching] The caching
   * type. Possible values include: 'None', 'ReadOnly', 'ReadWrite'
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.writeAcceleratorEnabled]
   * Specifies whether writeAccelerator should be enabled or disabled on the
   * disk.
   *
   * @param {number}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.diskSizeGB]
   * Specifies the size of the operating system disk in gigabytes. This element
   * can be used to overwrite the size of the disk in a virtual machine image.
   * <br><br> This value cannot be larger than 1023 GB
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image] The Source
   * User Image VirtualHardDisk. This VirtualHardDisk will be copied before using
   * it to attach to the Virtual Machine. If SourceImage is provided, the
   * destination VirtualHardDisk should not exist.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.image.uri] Specifies
   * the virtual hard disk's uri.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.vhdContainers] The
   * list of virtual hard disk container uris.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk] The
   * managed disk parameters.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.storageProfile.osDisk.managedDisk.storageAccountType]
   * Specifies the storage account type for the managed disk. NOTE: UltraSSD_LRS
   * can only be used with data disks, it cannot be used with OS Disk. Possible
   * values include: 'Standard_LRS', 'Premium_LRS', 'StandardSSD_LRS',
   * 'UltraSSD_LRS'
   *
   * @param {array} [parameters.virtualMachineProfile.storageProfile.dataDisks]
   * The data disks.
   *
   * @param {object} [parameters.virtualMachineProfile.networkProfile] The
   * virtual machine scale set network profile.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.networkProfile.networkInterfaceConfigurations]
   * The list of network configurations.
   *
   * @param {object} [parameters.virtualMachineProfile.diagnosticsProfile] The
   * virtual machine scale set diagnostics profile.
   *
   * @param {object}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics] Boot
   * Diagnostics is a debugging feature which allows you to view Console Output
   * and Screenshot to diagnose VM status. <br><br> You can easily view the
   * output of your console log. <br><br> Azure also enables you to see a
   * screenshot of the VM from the hypervisor.
   *
   * @param {boolean}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.enabled]
   * Whether boot diagnostics should be enabled on the Virtual Machine.
   *
   * @param {string}
   * [parameters.virtualMachineProfile.diagnosticsProfile.bootDiagnostics.storageUri]
   * Uri of the storage account to use for placing the console output and
   * screenshot.
   *
   * @param {object} [parameters.virtualMachineProfile.extensionProfile] The
   * virtual machine scale set extension profile.
   *
   * @param {array}
   * [parameters.virtualMachineProfile.extensionProfile.extensions] The virtual
   * machine scale set child extension resources.
   *
   * @param {string} [parameters.virtualMachineProfile.licenseType] The license
   * type, which is for bring your own license scenario.
   *
   * @param {boolean} [parameters.overprovision] Specifies whether the Virtual
   * Machine Scale Set should be overprovisioned.
   *
   * @param {boolean} [parameters.singlePlacementGroup] When true this limits the
   * scale set to a single placement group, of max size 100 virtual machines.
   *
   * @param {object} [parameters.identity] The identity of the virtual machine
   * scale set, if configured.
   *
   * @param {string} [parameters.identity.type] The type of identity used for the
   * virtual machine scale set. The type 'SystemAssigned, UserAssigned' includes
   * both an implicitly created identity and a set of user assigned identities.
   * The type 'None' will remove any identities from the virtual machine scale
   * set. Possible values include: 'SystemAssigned', 'UserAssigned',
   * 'SystemAssigned, UserAssigned', 'None'
   *
   * @param {object} [parameters.identity.userAssignedIdentities] The list of
   * user identities associated with the virtual machine scale set. The user
   * identity dictionary key references will be ARM resource ids in the form:
   * '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{identityName}'.
   *
   * @param {object} [parameters.tags] Resource tags
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSet} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSet} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginUpdate(resourceGroupName, vmScaleSetName, parameters, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginUpdate(resourceGroupName, vmScaleSetName, parameters, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginUpdate(resourceGroupName, vmScaleSetName, parameters, options, optionalCallback);
    }
  }

  /**
   * Deletes a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginDeleteMethodWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginDeleteMethod(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginDeleteMethod(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginDeleteMethod(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginDeleteMethod(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Deallocates specific virtual machines in a VM scale set. Shuts down the
   * virtual machines and releases the compute resources. You are not billed for
   * the compute resources that this virtual machine scale set deallocates.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginDeallocateWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginDeallocate(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deallocates specific virtual machines in a VM scale set. Shuts down the
   * virtual machines and releases the compute resources. You are not billed for
   * the compute resources that this virtual machine scale set deallocates.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginDeallocate(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginDeallocate(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginDeallocate(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Deletes virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {array} instanceIds The virtual machine scale set instance ids.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginDeleteInstancesWithHttpOperationResponse(resourceGroupName, vmScaleSetName, instanceIds, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginDeleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Deletes virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {array} instanceIds The virtual machine scale set instance ids.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginDeleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginDeleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginDeleteInstances(resourceGroupName, vmScaleSetName, instanceIds, options, optionalCallback);
    }
  }

  /**
   * Power off (stop) one or more virtual machines in a VM scale set. Note that
   * resources are still attached and you are getting charged for the resources.
   * Instead, use deallocate to release resources and avoid charges.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginPowerOffWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginPowerOff(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Power off (stop) one or more virtual machines in a VM scale set. Note that
   * resources are still attached and you are getting charged for the resources.
   * Instead, use deallocate to release resources and avoid charges.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginPowerOff(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginPowerOff(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginPowerOff(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Restarts one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginRestartWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginRestart(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Restarts one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginRestart(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginRestart(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginRestart(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Starts one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginStartWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginStart(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Starts one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginStart(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginStart(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginStart(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Redeploy one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginRedeployWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginRedeploy(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Redeploy one or more virtual machines in a VM scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginRedeploy(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginRedeploy(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginRedeploy(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Perform maintenance on one or more virtual machines in a VM scale set.
   * Operation on instances which are not eligible for perform maintenance will
   * be failed. Please refer to best practices for more details:
   * https://docs.microsoft.com/en-us/azure/virtual-machine-scale-sets/virtual-machine-scale-sets-maintenance-notifications
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginPerformMaintenanceWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginPerformMaintenance(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Perform maintenance on one or more virtual machines in a VM scale set.
   * Operation on instances which are not eligible for perform maintenance will
   * be failed. Please refer to best practices for more details:
   * https://docs.microsoft.com/en-us/azure/virtual-machine-scale-sets/virtual-machine-scale-sets-maintenance-notifications
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginPerformMaintenance(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginPerformMaintenance(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginPerformMaintenance(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Upgrades one or more virtual machines to the latest SKU set in the VM scale
   * set model.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {array} instanceIds The virtual machine scale set instance ids.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginUpdateInstancesWithHttpOperationResponse(resourceGroupName, vmScaleSetName, instanceIds, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginUpdateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Upgrades one or more virtual machines to the latest SKU set in the VM scale
   * set model.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {array} instanceIds The virtual machine scale set instance ids.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginUpdateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginUpdateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginUpdateInstances(resourceGroupName, vmScaleSetName, instanceIds, options, optionalCallback);
    }
  }

  /**
   * Reimages (upgrade the operating system) one or more virtual machines in a VM
   * scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginReimageWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginReimage(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Reimages (upgrade the operating system) one or more virtual machines in a VM
   * scale set.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginReimage(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginReimage(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginReimage(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Reimages all the disks ( including data disks ) in the virtual machines in a
   * VM scale set. This operation is only supported for managed disks.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<null>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  beginReimageAllWithHttpOperationResponse(resourceGroupName, vmScaleSetName, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._beginReimageAll(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Reimages all the disks ( including data disks ) in the virtual machines in a
   * VM scale set. This operation is only supported for managed disks.
   *
   * @param {string} resourceGroupName The name of the resource group.
   *
   * @param {string} vmScaleSetName The name of the VM scale set.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {array} [options.instanceIds] The virtual machine scale set instance
   * ids. Omitting the virtual machine scale set instance ids will result in the
   * operation being performed on all virtual machines in the virtual machine
   * scale set.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {null} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {null} [result]   - The deserialized result object if an error did not occur.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  beginReimageAll(resourceGroupName, vmScaleSetName, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._beginReimageAll(resourceGroupName, vmScaleSetName, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._beginReimageAll(resourceGroupName, vmScaleSetName, options, optionalCallback);
    }
  }

  /**
   * Gets a list of all VM scale sets under a resource group.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetListResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listNextWithHttpOperationResponse(nextPageLink, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._listNext(nextPageLink, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets a list of all VM scale sets under a resource group.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetListResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetListResult} for more
   *                      information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  listNext(nextPageLink, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._listNext(nextPageLink, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._listNext(nextPageLink, options, optionalCallback);
    }
  }

  /**
   * Gets a list of all VM Scale Sets in the subscription, regardless of the
   * associated resource group. Use nextLink property in the response to get the
   * next page of VM Scale Sets. Do this till nextLink is null to fetch all the
   * VM Scale Sets.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetListWithLinkResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listAllNextWithHttpOperationResponse(nextPageLink, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._listAllNext(nextPageLink, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets a list of all VM Scale Sets in the subscription, regardless of the
   * associated resource group. Use nextLink property in the response to get the
   * next page of VM Scale Sets. Do this till nextLink is null to fetch all the
   * VM Scale Sets.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetListWithLinkResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetListWithLinkResult}
   *                      for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  listAllNext(nextPageLink, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._listAllNext(nextPageLink, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._listAllNext(nextPageLink, options, optionalCallback);
    }
  }

  /**
   * Gets a list of SKUs available for your VM scale set, including the minimum
   * and maximum VM instances allowed for each SKU.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetListSkusResult>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  listSkusNextWithHttpOperationResponse(nextPageLink, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._listSkusNext(nextPageLink, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets a list of SKUs available for your VM scale set, including the minimum
   * and maximum VM instances allowed for each SKU.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetListSkusResult} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetListSkusResult} for
   *                      more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  listSkusNext(nextPageLink, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._listSkusNext(nextPageLink, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._listSkusNext(nextPageLink, options, optionalCallback);
    }
  }

  /**
   * Gets list of OS upgrades on a VM scale set instance.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @returns {Promise} A promise is returned
   *
   * @resolve {HttpOperationResponse<VirtualMachineScaleSetListOSUpgradeHistory>} - The deserialized result object.
   *
   * @reject {Error} - The error object.
   */
  getOSUpgradeHistoryNextWithHttpOperationResponse(nextPageLink, options) {
    let client = this.client;
    let self = this;
    return new Promise((resolve, reject) => {
      self._getOSUpgradeHistoryNext(nextPageLink, options, (err, result, request, response) => {
        let httpOperationResponse = new msRest.HttpOperationResponse(request, response);
        httpOperationResponse.body = result;
        if (err) { reject(err); }
        else { resolve(httpOperationResponse); }
        return;
      });
    });
  }

  /**
   * Gets list of OS upgrades on a VM scale set instance.
   *
   * @param {string} nextPageLink The NextLink from the previous successful call
   * to List operation.
   *
   * @param {object} [options] Optional Parameters.
   *
   * @param {object} [options.customHeaders] Headers that will be added to the
   * request
   *
   * @param {function} [optionalCallback] - The optional callback.
   *
   * @returns {function|Promise} If a callback was passed as the last parameter
   * then it returns the callback else returns a Promise.
   *
   * {Promise} A promise is returned
   *
   *                      @resolve {VirtualMachineScaleSetListOSUpgradeHistory} - The deserialized result object.
   *
   *                      @reject {Error} - The error object.
   *
   * {function} optionalCallback(err, result, request, response)
   *
   *                      {Error}  err        - The Error object if an error occurred, null otherwise.
   *
   *                      {object} [result]   - The deserialized result object if an error did not occur.
   *                      See {@link VirtualMachineScaleSetListOSUpgradeHistory}
   *                      for more information.
   *
   *                      {object} [request]  - The HTTP Request object if an error did not occur.
   *
   *                      {stream} [response] - The HTTP Response stream if an error did not occur.
   */
  getOSUpgradeHistoryNext(nextPageLink, options, optionalCallback) {
    let client = this.client;
    let self = this;
    if (!optionalCallback && typeof options === 'function') {
      optionalCallback = options;
      options = null;
    }
    if (!optionalCallback) {
      return new Promise((resolve, reject) => {
        self._getOSUpgradeHistoryNext(nextPageLink, options, (err, result, request, response) => {
          if (err) { reject(err); }
          else { resolve(result); }
          return;
        });
      });
    } else {
      return self._getOSUpgradeHistoryNext(nextPageLink, options, optionalCallback);
    }
  }

}

module.exports = VirtualMachineScaleSets;
